import logging
import re
from types import FunctionType as function

from App.class_init import InitializeClass
from Products.Archetypes.log import log
from Products.Archetypes.utils import capitalize
from Products.Archetypes.utils import _getSecurity

# marker that AT should generate a method -- used to discard unwanted
#  inherited methods
AT_GENERATE_METHOD = []


_modes = {
    'r': {'prefix': 'get',
          'attr': 'accessor',
          'security': 'read_permission',
          },
    'm': {'prefix': 'getRaw',
          'attr': 'edit_accessor',
          'security': 'write_permission',
          },
    'w': {'prefix': 'set',
          'attr': 'mutator',
          'security': 'write_permission',
          },
    }


class GeneratorError(Exception):
    pass


class Generator:
    def computeMethodName(self, field, mode):
        if mode not in _modes.keys():
            raise TypeError("Unsupported Mode %s in field: %s (%s)" % \
                            (field.getName(), mode))

        prefix = _modes[mode]['prefix']
        name = capitalize(field.getName())
        return "%s%s" % (prefix, name)

    def makeMethod(self, klass, field, mode, methodName):
        name = field.getName()
        method = None
        if mode == "r":
            def generatedAccessor(self, **kw):
                # Default Accessor.
                if 'schema' in kw:
                    schema = kw['schema']
                else:
                    schema = self.Schema()
                    kw['schema'] = schema
                return schema[name].get(self, **kw)
            method = generatedAccessor
        elif mode == "m":
            def generatedEditAccessor(self, **kw):
                # Default Edit Accessor.
                if 'schema' in kw:
                    schema = kw['schema']
                else:
                    schema = self.Schema()
                    kw['schema'] = schema
                return schema[name].getRaw(self, **kw)
            method = generatedEditAccessor
        elif mode == "w":
            def generatedMutator(self, value, **kw):
                # Default Mutator.
                if 'schema' in kw:
                    schema = kw['schema']
                else:
                    schema = self.Schema()
                    kw['schema'] = schema
                return schema[name].set(self, value, **kw)
            method = generatedMutator
        else:
            raise GeneratorError("""Unhandled mode for method creation:
            %s:%s -> %s:%s""" % (klass.__name__,
                                 name,
                                 methodName,
                                 mode))

        # Zope security requires all security protected methods to have a
        # function name. It uses this name to determine which roles are allowed
        # to access the method.
        # This code is renaming the internal name from e.g. generatedAccessor to
        # methodName.
        method = function(method.func_code,
                          method.func_globals,
                          methodName,
                          method.func_defaults,
                          method.func_closure,
                         )
        setattr(klass, methodName, method)


class ClassGenerator:
    def updateSecurity(self, klass, field, mode, methodName):
        security = _getSecurity(klass)

        perm = _modes[mode]['security']
        perm = getattr(field, perm, None)
        method__roles__ = getattr(klass, '%s__roles__' % methodName, 0)

        # Check copied from SecurityInfo to avoid stomping over
        # existing permissions.
        if security.names.get(methodName, perm) != perm:
            log('The method \'%s\' was already protected by a '
                'different permission than the one declared '
                'on the field. Assuming that the explicit '
                'permission declared is the correct one and '
                'has preference over the permission declared '
                'on the field.' % methodName, level=logging.DEBUG)
        elif method__roles__ is None:
            security.declarePublic(methodName)
        elif method__roles__ == ():
            security.declarePrivate(methodName)
        else:
            security.declareProtected(perm, methodName)

    def generateName(self, klass):
        return re.sub('([a-z])([A-Z])', '\g<1> \g<2>', klass.__name__)

    def checkSchema(self, klass):
        pass

    def generateClass(self, klass):
        # We are going to assert a few things about the class here
        # before we start, set meta_type, portal_type based on class
        # name, but only if they are not set yet
        if (not getattr(klass, 'meta_type', None) or
            'meta_type' not in klass.__dict__.keys()):
            klass.meta_type = klass.__name__
        if (not getattr(klass, 'portal_type', None) or
            'portal_type' not in klass.__dict__.keys()):
            klass.portal_type = klass.__name__
        klass.archetype_name = getattr(klass, 'archetype_name',
                                       self.generateName(klass))

        self.checkSchema(klass)
        fields = klass.schema.fields()
        self.generateMethods(klass, fields)

    def generateMethods(self, klass, fields):
        generator = Generator()
        for field in fields:
            assert not 'm' in field.mode, 'm is an implicit mode'

            # Make sure we want to muck with the class for this field
            if "c" not in field.generateMode: continue
            type = getattr(klass, 'schema')
            for mode in field.mode:  # (r, w)
                self.handle_mode(klass, generator, type, field, mode)
                if mode == 'w':
                    self.handle_mode(klass, generator, type, field, 'm')

        InitializeClass(klass)

    def handle_mode(self, klass, generator, type, field, mode):
        attr = _modes[mode]['attr']
        # Did the field request a specific method name?
        methodName = getattr(field, attr, None)
        if not methodName:
            methodName = generator.computeMethodName(field, mode)

        # Avoid name space conflicts
        if not hasattr(klass, methodName) \
               or getattr(klass, methodName) is AT_GENERATE_METHOD:
            if methodName in type:
                raise GeneratorError("There is a conflict "
                                     "between the Field(%s) and the attempt "
                                     "to generate a method of the same name on "
                                     "class %s" % (
                    methodName,
                    klass.__name__))

            # Make a method for this klass/field/mode
            generator.makeMethod(klass, field, mode, methodName)

        # Update security regardless of the method being generated or
        # not. Not protecting the method by the permission defined on
        # the field may leave security open and lead to misleading
        # bugs.
        self.updateSecurity(klass, field, mode, methodName)

        # Note on the class what we did (even if the method existed)
        attr = _modes[mode]['attr']
        setattr(field, attr, methodName)


def generateCtor(name, module):
    # self is a App.FactoryDispater, Destination() is the real folder
    ctor = """
def add%(name)s(self, id, **kwargs):
    obj = %(name)s(id)
    self._setObject(id, obj, suppress_events=True)
    obj = self._getOb(id)
    obj.manage_afterAdd(obj, self)
    obj.initializeArchetype(**kwargs)
    return obj.getId()
""" % {'name': name}
    exec ctor in module.__dict__
    return getattr(module, "add%s" % name)


def generateZMICtor(name, module):
    zmi_ctor = """
def manage_add%(name)s(self, id, REQUEST=None):
    ''' Constructor for %(name)s '''
    kwargs = {}
    if REQUEST is not None:
        kwargs = REQUEST.form.copy()
        del kwargs['id']
    id = add%(name)s(self, id, **kwargs)
    obj = self._getOb(id)
    manage_tabs_message = 'Successfully added %(name)s'
    if REQUEST is not None:
        url = obj.absolute_url()
        REQUEST.RESPONSE.redirect(url + '/manage_edit%(name)sForm?manage_tabs_message=' + manage_tabs_message)
    return id
""" % {'name': name}
    exec zmi_ctor in module.__dict__
    return getattr(module, "manage_add%s" % name)


_cg = ClassGenerator()
generateClass = _cg.generateClass
generateMethods = _cg.generateMethods



from Products.Archetypes.interfaces.layer import ILayer

class IMarshall(ILayer):
    """De/Marshall data.
    """

    def demarshall(instance, data, **kwargs):
        """Given the blob 'data' demarshall it and modify 'instance'
        accordingly if possible
        """

    def marshall(instance, **kwargs):
        """Returns a tuple of content-type, length, and data
        """



from Products.Archetypes.interfaces.layer import ILayerContainer
from ExtensionClass import Base
from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from zope.interface import implements

class DefaultLayerContainer(Base):
    implements(ILayerContainer)

    security = ClassSecurityInfo()

    def __init__(self):
        self._layers = {}

    security.declarePrivate('registerLayer')
    def registerLayer(self, name, object):
        self._layers[name] = object

    security.declarePrivate('registeredLayers')
    def registeredLayers(self):
        return self._layers.items()

    security.declarePrivate('hasLayer')
    def hasLayer(self, name):
        return name in self._layers.keys()

    security.declarePrivate('getLayerImpl')
    def getLayerImpl(self, name):
        return self._layers[name]

InitializeClass(DefaultLayerContainer)



""" \
Bring generic Ape Support to Archetypes.

The goal of this module is to implement generic mapping of Archetypes Schema
to real tables with real columns for each field in the schema.

**Experimental**

This code works with Ape 1.0

Whats working so far:

 - The following types are handled so far:

  - string,

  - int.

Whats not working so far:

 - References,images are not yet supported

 - Renaming of objects generates errors

ArchGenXML has support for APE:

 When you invoke ArchGenXML with the option --ape-support the outline_od.xmi
 sample works with APE correctly all ape_config and the serializer/gateway
 stuff is generated for you.

ApeSupport is tested with Ape 1.0 and PostgreSQL
"""

from Products.Archetypes.BaseUnit import BaseUnit
from Products.Archetypes.atapi import Master, __builtin__
from types import ClassType

from apelib.core.interfaces import ISerializer, IFullSerializationEvent
from apelib.sql.structure import RowSequenceSchema
from apelib.sql.properties import SQLFixedProperties
from apelib.zodb3.serializers import RemainingState as RemainingBase
from apelib.zodb3.serializers import encode_to_text

from Persistence import Persistent, PersistentMapping
from StringIO import StringIO
from cPickle import Pickler, UnpickleableError
import os

from zope.interface import implements

# map types between APE and Archetypes Schemas

typemap = {
    'text': 'string',
    'datetime': 'string',
    'boolean': 'int',
    'integer': 'int',
    #'reference': 'string:list',
    'computed': 'string'  # ouch!!
    }


def AtType2ApeType(f):
    t = f._properties['type']
    if t == 'reference':
        #print 'REF:',f.getName(),f.multiValued
        if f.multiValued:
            return 'string'
        else:
            return 'string'
    if t == 'computed':
        return None

    return typemap.get(t, t)


def AtSchema2ApeSchema(atschema):
    schema = RowSequenceSchema()
    column_defs = []
    for f in atschema.fields():
        if f.isMetadata:
            continue
        pk = 0
        name = f.getName()
        t = AtType2ApeType(f)
        if not t:  # then dont add it to the schema
            continue
        if name == 'id':
            pk = 1
        schema.add(name, t, pk)
        column_defs.append((name, t, pk))

    #print schema, tuple(column_defs)
    return schema, tuple(column_defs)


# creates a generic gateway instance based on
# the klass's Schema
def constructGateway(klass):
    table_name = klass.__name__.lower()
    schema, column_defs = AtSchema2ApeSchema(klass.schema)
    res = SQLFixedProperties('db', table_name, schema)
    return res


# creates a generic serializer instance based on
# the klass's Schema
def constructSerializer(klass):
    res = ArcheSerializer()
    res.klass = klass
    res.schema = AtSchema2ApeSchema(klass.schema)[0]
    return res


# generic Serializer class
# which reflects the class's Schema
class ArcheSerializer:
    """Serializer for OFS.PropertyManager properties."""

    implements(ISerializer)

    schema = RowSequenceSchema()

    def getSchema(self):
        return self.schema

    def can_serialize(self, object):
        return isinstance(object, Master)

    def serialize(self, event):
        res = []
        for f in event.obj.Schema().fields():
            if f.isMetadata:
                continue

            name = f.getName()
            t = AtType2ApeType(f)
            if not t:
                continue
            event.ignore(name)
            data = f.getAccessor(event.obj)()
            res.append((name, t, data))
        return res

    def deserialize(self, event, state):
        for id, t, v in state:
            event.obj.__dict__.update({id: v})


# this replacement of RemainingState is necessary in order to
# replace the BaseUnit members by string data because
# Baseunits are not pickleable (dunno why)
# overloaded the serialize method in order to clean the __dict__
# correctly
class RemainingState(RemainingBase):

    def cleanDictCopy(self, dict):
        ''' cleans out the baseUnit instances of the dict, because the are not picklable '''
        res = {}

        for k in dict.keys():
            v = dict[k]
            if type(v) == type({}) or ec_isinstance(v, PersistentMapping):
                v1 = self.cleanDictCopy(v)
            elif ec_isinstance(v, BaseUnit):
                v1 = v.getRaw()
            else:
                v1 = v
            res[k] = v1

        return res

    def serialize(self, event):
        assert IFullSerializationEvent.providedBy(event)
        assert isinstance(event.obj, Persistent)

        # Allow pickling of cyclic references to the object.
        event.serialized('self', event.obj, False)

        # Ignore previously serialized attributes
        state = self.cleanDictCopy(event.obj.__dict__)
        for key in state.keys():
            if key.startswith('_v_'):
                del state[key]
        for attrname in event.get_seralized_attributes():
            if attrname in state:
                del state[attrname]
        if not state:
            # No data needs to be stored
            return ''

        outfile = StringIO()
        p = Pickler(outfile, 1)  # Binary pickle
        unmanaged = []

        def persistent_id(ob, identify_internal=event.identify_internal,
                          unmanaged=unmanaged):
            ref = identify_internal(ob)
            if ref is None:
                if hasattr(ob, '_p_oid'):
                    # Persistent objects that end up in the remainder
                    # are unmanaged.  Tell ZODB about them so that
                    # ZODB can deal with them specially.
                    unmanaged.append(ob)
            return ref

        # Preserve order to a reasonable extent by storing a list
        # instead of a dictionary.
        state_list = state.items()
        state_list.sort()
        p.persistent_id = persistent_id
        try:
            p.dump(state_list)
        except UnpickleableError, exc:
            # Try to reveal which attribute is unpickleable.
            attrname = None
            attrvalue = None
            for key, value in state_list:
                del unmanaged[:]
                outfile.seek(0)
                outfile.truncate()
                p = Pickler(outfile)
                p.persistent_id = persistent_id
                try:
                    p.dump(value)
                except UnpickleableError:
                    attrname = key
                    attrvalue = value
                    break
            if attrname is not None:
                # Provide a more informative exception.
                if os.environ.get('APE_TRACE_UNPICKLEABLE'):
                    # Provide an opportunity to examine
                    # the "attrvalue" attribute.
                    raise RuntimeError(
                        'Unable to pickle the %s attribute, %s, '
                        'of %s at %s.  %s.' % (
                        repr(attrname), repr(attrvalue), repr(event.obj),
                        repr(event.oid), str(exc)))
            else:
                # Couldn't help.
                raise

        p.persistent_id = lambda ob: None  # Stop recording references
        p.dump(unmanaged)
        event.upos.extend(unmanaged)

        s = outfile.getvalue()
        return encode_to_text(s, state.keys(), len(unmanaged))


# helper functions for issubclass and isinstance
# with extension classes.
# borrowed from Greg Ward (thanx Greg :)
def ec_issubclass(class1, class2):
    """A version of 'issubclass' that works with extension classes
    as well as regular Python classes.
    """

    # Both class objects are regular Python classes, so use the
    # built-in 'issubclass()'.
    if type(class1) is ClassType and type(class2) is ClassType:
        return __builtin__.issubclass(class1, class2)

    # Both so-called class objects have a '__bases__' attribute: ie.,
    # they aren't regular Python classes, but they sure look like them.
    # Assume they are extension classes and reimplement what the builtin
    # 'issubclass()' does behind the scenes.
    elif hasattr(class1, '__bases__') and hasattr(class2, '__bases__'):
        # XXX it appears that "ec.__class__ is type(ec)" for an
        # extension class 'ec': could we/should we use this as an
        # additional check for extension classes?

        # Breadth-first traversal of class1's superclass tree.  Order
        # doesn't matter because we're just looking for a "yes/no"
        # answer from the tree; if we were trying to resolve a name,
        # order would be important!
        stack = [class1]
        while stack:
            if stack[0] is class2:
                return 1
            stack.extend(list(stack[0].__bases__))
            del stack[0]
        else:
            return 0

    # Not a regular class, not an extension class: blow up for consistency
    # with builtin 'issubclass()"
    else:
        raise TypeError, "arguments must be class or ExtensionClass objects"

# ec_issubclass ()


def ec_isinstance(object, klass):
    """A version of 'isinstance' that works with extension classes
    as well as regular Python classes."""

    if type(klass) is ClassType:
        return isinstance(object, klass)
    elif hasattr(object, '__class__'):
        return ec_issubclass(object.__class__, klass)
    else:
        return 0



################################################################################
#
# Copyright (c) 2002-2005, Benjamin Saller <bcsaller@ideasuite.com>, and
#                              the respective authors. All rights reserved.
# For a list of Archetypes contributors see docs/CREDITS.txt.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the author nor the names of its contributors may be used
#   to endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
################################################################################

from Products.Archetypes.tests.atsitetestcase import ATSiteTestCase
from Products.Archetypes.tests.utils import makeContent

from Products.Archetypes.atapi import *
from Products.Archetypes.interfaces.base import IBaseUnit


class ExternalEditorTest(ATSiteTestCase):

    def testExternalEditor(self):
        #really a test that baseobject.__getitem__ returns something
        #which externaleditor can use
        obj = makeContent(self.folder, portal_type='SimpleType', id='obj')
        self.assertTrue(IBaseUnit.providedBy(obj.body))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(ExternalEditorTest))
    return suite



################################################################################
#
# Copyright (c) 2002-2005, Benjamin Saller <bcsaller@ideasuite.com>, and
#                              the respective authors. All rights reserved.
# For a list of Archetypes contributors see docs/CREDITS.txt.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the author nor the names of its contributors may be used
#   to endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
################################################################################

from unittest import TestSuite, makeSuite

from Products.Archetypes.tests.atsitetestcase import ATSiteTestCase
from Products.Archetypes.atapi import BaseSchema, Schema, StringField, \
    AnnotationStorage, MetadataAnnotationStorage, AT_MD_STORAGE, \
    getAnnotation, AT_ANN_STORAGE
from Products.Archetypes.tests.test_classgen import Dummy
from Products.Archetypes.tests.test_classgen import gen_class
from Products.Archetypes.tests.test_classgen import gen_dummy
from Acquisition import aq_base


class AnnDummy(Dummy): pass

annschema = BaseSchema + Schema((
     StringField('string',
         default=u'stringdefault',
         storage=AnnotationStorage(),
         ),
     StringField('meta',
         default='metadefault',
         storage=MetadataAnnotationStorage(),
         ),
    ))


def gen_anndummy():
    gen_class(AnnDummy, annschema)


class AnnotationTest(ATSiteTestCase):

    def afterSetUp(self):
        gen_dummy()
        dummy = Dummy(oid='dummy')
        self.folder._setObject('dummy', dummy)
        self.folder.dummy.initializeArchetype()
        self.dummy = self.folder.dummy
        self.ann = getAnnotation(self.dummy)

    def test_empty(self):
        ann = self.ann
        self.assertEqual(bool(ann), False)
        self.assertEqual(getattr(self.dummy, '__annotations__', None), None)

    def test_create(self):
        ann = self.ann
        ann['test'] = 'test'
        self.assertEqual(bool(ann), True)
        self.assertNotEqual(getattr(self.dummy, '__annotations__', None), None)

    def test_set(self):
        ann = self.ann
        ann['test'] = 'test1'
        self.assertEqual(ann['test'], 'test1')
        ann.set('test', 'test2')
        self.assertEqual(ann['test'], 'test2')
        ann.setSubkey('test', 'test3', subkey='testsub')
        self.assertEqual(ann['test-testsub'], 'test3')
        self.assertEqual(ann.getSubkey('test', subkey='testsub'), 'test3')

    def test_get(self):
        ann = self.ann
        ann['test'] = 'test1'
        self.assertEqual(ann['test'], 'test1')
        self.assertEqual(ann.get('test'), 'test1')
        self.assertTrue('test' in ann)
        self.assertEqual(ann.get('none', default='default'), 'default')

    def test_del(self):
        ann = self.ann
        ann['test'] = 'test1'
        del ann['test']
        self.assertFalse('test' in ann)
        ann.setSubkey('test', 'test3', subkey='testsub')
        ann.delSubkey('test', subkey='testsub')
        self.assertFalse('test-testsub' in ann)
        self.assertFalse(ann.hasSubkey('test', subkey='testsub'))


class MetadataAnnotationStorageTest(ATSiteTestCase):

    def afterSetUp(self):
        gen_anndummy()
        dummy = AnnDummy(oid='dummy')
        self.folder._setObject('dummy', dummy)
        self.dummy = self.folder.dummy
        self.dummy.initializeArchetype()
        self.ann = getAnnotation(self.dummy)

    def test_setup(self):
        dummy = self.dummy
        field = dummy.getField('meta')
        self.assertTrue(isinstance(field.storage, MetadataAnnotationStorage))
        self.assertTrue(self.ann.hasSubkey(AT_MD_STORAGE, 'meta'))
        self.assertEqual(self.ann.getSubkey(AT_MD_STORAGE, subkey='meta'), 'metadefault')

    def test_gestset(self):
        dummy = self.dummy
        ann = self.ann
        dummy.setMeta('egg')
        self.assertEqual(dummy.getMeta(), 'egg')
        self.assertEqual(ann.getSubkey(AT_MD_STORAGE, subkey='meta'), 'egg')


class AnnotationStorageTest(ATSiteTestCase):

    def afterSetUp(self):
        gen_anndummy()
        dummy = AnnDummy(oid='dummy')
        self.folder._setObject('dummy', dummy)
        self.dummy = self.folder.dummy
        self.dummy.initializeArchetype()
        self.ann = getAnnotation(self.dummy)

    def test_setup(self):
        dummy = self.dummy
        field = dummy.getField('string')
        self.assertTrue(isinstance(field.storage, AnnotationStorage))
        self.assertTrue(self.ann.hasSubkey(AT_ANN_STORAGE, 'string'))
        self.assertEqual(self.ann.getSubkey(AT_ANN_STORAGE, subkey='string'), 'stringdefault')

    def test_gestset(self):
        dummy = self.dummy
        ann = self.ann
        dummy.setString('egg')
        self.assertEqual(dummy.getString(), 'egg')
        self.assertEqual(ann.getSubkey(AT_ANN_STORAGE, subkey='string'), 'egg')

    def test_storageGetSetDel(self):
        dummy = self.dummy
        ann = self.ann
        field = dummy.getField('string')
        storage = field.storage

        dummy.setString('egg')
        self.assertEqual(storage.get('string', dummy), 'egg')

        storage.set('string', dummy, 'spam')
        self.assertEqual(storage.get('string', dummy), 'spam')

        storage.unset('string', dummy)
        self.assertFalse(ann.hasSubkey(AT_ANN_STORAGE, 'string'))

    def test_storageNonExisting(self):
        dummy = self.dummy
        ann = self.ann
        storage = AnnotationStorage()

        self.assertRaises(AttributeError, storage.get, 'nonexisting', dummy)

        # del shouldn't raise anything
        storage.unset('nonexisting', dummy)

        # set should create an entry
        storage.set('nonexisting', dummy,  value='bar')
        self.assertEqual(storage.get('nonexisting', dummy), 'bar')

    def test_migration(self):
        dummy = self.dummy
        ann = self.ann
        field = dummy.getField('string')
        storage = field.storage

        # migration mode on
        self.assertEqual(storage._migrate, False)
        storage._migrate = True

        # test clean up
        dummy.string = 'spam'
        self.assertTrue(hasattr(aq_base(dummy), 'string'))

        self.assertNotEqual(storage.get('string', dummy), 'spam')
        storage.set('string', dummy, 'bar')
        self.assertFalse(hasattr(aq_base(dummy), 'string'))
        self.assertEqual(storage.get('string', dummy), 'bar')

        # test migration
        ann.delSubkey(AT_ANN_STORAGE, subkey='string')
        dummy.string = 'spam'
        self.assertEqual(storage.get('string', dummy), 'spam')
        self.assertFalse(hasattr(aq_base(dummy), 'string'))


def test_suite():
    suite = TestSuite()
    suite.addTest(makeSuite(AnnotationTest))
    suite.addTest(makeSuite(MetadataAnnotationStorageTest))
    suite.addTest(makeSuite(AnnotationStorageTest))
    return suite



# -*- coding: utf-8 -*-
################################################################################
#
# Copyright (c) 2002-2005, Benjamin Saller <bcsaller@ideasuite.com>, and
#                              the respective authors. All rights reserved.
# For a list of Archetypes contributors see docs/CREDITS.txt.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the author nor the names of its contributors may be used
#   to endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
################################################################################
"""
"""


from Products.Archetypes.tests.attestcase import ATTestCase
from Products.Archetypes.utils import DisplayList
from Products.Archetypes.utils import IntDisplayList
from Products.Archetypes.utils import Vocabulary
from Products.Archetypes.utils import make_uuid


class UidGeneratorTest(ATTestCase):
    """Some ppl have reported problems with uids. This test isn't mathematical
    correct but should show the issue on plattform. I suspect it's Windows :|
    """

    def test_uuid(self):
        uids = {}
        loop_length = 10 ** 5  # about 1.5 seconds on a fast cpu
        for i in xrange(loop_length):
            uid = make_uuid()
            uids[uid] = 1
        self.assertEqual(len(uids), loop_length)


class DisplayListTest(ATTestCase):

    def test_cmp(self):
        ta = ('a', 'b', 'c')
        tb = ('a', 'c', 'b')
        td = ('c', 'b', 'a')
        self.assertTrue(DisplayList(zip(ta, ta)) == DisplayList(zip(ta, ta)))
        self.assertFalse(DisplayList(zip(ta, ta)) == DisplayList(zip(ta, tb)))
        self.assertTrue(DisplayList(zip(ta, ta)) == DisplayList(zip(td, td)))
        self.assertTrue(DisplayList(zip(tb, ta)) == DisplayList(zip(tb, ta)))
        self.assertRaises(TypeError, cmp, DisplayList(), '')

    def test_slice(self):
        ta = ('a', 'b', 'c')
        l = zip(ta, ta)
        sub = l[1:]
        self.assertTrue(DisplayList(l)[1:] == sub)

    def test_item(self):
        ta = ('a', 'b', 'c')
        l = zip(ta, ta)
        for i in range(0, 2):
            item = ta[i]
            self.assertTrue(DisplayList(l)[i] == item)

    def test_add(self):
        ta = ('a', 'b', 'c')
        l = zip(ta, ta)
        dl = DisplayList(l)[:]
        self.assertTrue(dl == l)
        l.append(('d', 'd'))
        dl.append(('d', 'd'))
        self.assertTrue(dl == l)

    def test_len(self):
        ta = ('a', 'b', 'c')
        l = zip(ta, ta)
        dl = DisplayList(l)
        self.assertTrue(len(dl) == len(l))

    def test_keys(self):
        ta = ('a', 'b', 'c')
        l = zip(ta, ta)
        dl = DisplayList(l)
        self.assertTrue(tuple(dl.keys()) == ta)

    def test_values(self):
        ta = ('a', 'b', 'c')
        l = zip(ta, ta)
        dl = DisplayList(l)
        self.assertTrue(tuple(dl.values()) == ta)

    def test_items(self):
        ta = ('a', 'b', 'c')
        l = zip(ta, ta)
        dl = DisplayList(l)
        self.assertTrue(dl.items() == tuple(l))

    def test_repr(self):
        ta = ('a', 'b', 'c')
        l = zip(ta, ta)
        dl = DisplayList(l)
        self.assertTrue(repr(dl).find(str(l)))

    def test_str(self):
        ta = ('a', 'b', 'c')
        l = zip(ta, ta)
        dl = DisplayList(l)
        self.assertTrue(str(dl) == str(l))

    def test_call(self):
        ta = ('a', 'b', 'c')
        l = zip(ta, ta)
        dl = DisplayList(l)
        self.assertTrue(dl == dl)
        self.assertTrue(dl() == dl())
        self.assertTrue(dl[:] == l)
        self.assertTrue(dl()[:] == l)

    def test_sort(self):
        a = (('a', 'a',), ('b', 'b'), ('c', 'c'))
        b = (('z', 'Z',), ('y', 'Y'), ('x', 'X'))
        c = (('a', 'Z',), ('c', 'Y'), ('b', 'X'))
        dla = DisplayList(a)
        dlb = DisplayList(b)
        dlc = DisplayList(c)

        assert dla.values() == ['a', 'b', 'c']
        dlb_s = dlb.sortedByValue()
        assert dlb_s.values() == ['X', 'Y', 'Z']
        dlc_s = dlc.sortedByKey()
        assert dlc_s.values() == ['Z', 'X', 'Y']

    def test_getValue(self):
        a = (('a', 'A',), ('b', '\xc3\xab'), ('c', u'xeb'), ('d', 42))
        dla = DisplayList(a)
        self.assertEqual(dla.getValue('a'), 'A')
        self.assertEqual(dla.getValue('b'), '\xc3\xab')
        self.assertEqual(dla.getValue('c'), u'xeb')
        self.assertEqual(dla.getValue('d'), 42)
        self.assertEqual(dla.getValue('e'), None)
        self.assertEqual(dla.getValue('e', 'default'), 'default')

        # Switch the keys and values around.
        b = (('A', 'a',), ('\xc3\xab', 'b'), (u'xeb', 'c'))
        dlb = DisplayList(b)
        self.assertEqual(dlb.getValue('A'), 'a')
        self.assertEqual(dlb.getValue('\xc3\xab'), 'b')
        self.assertEqual(dlb.getValue(u'xeb'), 'c')
        self.assertEqual(dlb.getValue('e'), None)
        self.assertEqual(dlb.getValue('e', 'default'), 'default')


class IntDisplayListTest(ATTestCase):

    def test_cmp(self):
        ta = (1, 2, 3)
        tb = (1, 3, 2)
        td = (3, 2, 1)
        self.assertTrue(IntDisplayList(zip(ta, ta)) == IntDisplayList(zip(ta, ta)))
        self.assertFalse(IntDisplayList(zip(ta, ta)) == IntDisplayList(zip(ta, tb)))
        self.assertTrue(IntDisplayList(zip(ta, ta)) == IntDisplayList(zip(td, td)))
        self.assertTrue(IntDisplayList(zip(tb, ta)) == IntDisplayList(zip(tb, ta)))
        self.assertRaises(TypeError, cmp, IntDisplayList(), '')

    def test_slice(self):
        ta = (1, 2, 3)
        l = zip(ta, ta)
        sub = l[1:]
        self.assertTrue(IntDisplayList(l)[1:] == sub)

    def test_item(self):
        ta = (1, 2, 3)
        l = zip(ta, ta)
        for i in range(0, 2):
            item = ta[i]
            self.assertTrue(IntDisplayList(l)[i] == item)

    def test_add(self):
        ta = (1, 2, 3)
        l = zip(ta, ta)
        dl = IntDisplayList(l)[:]
        self.assertTrue(dl == l)
        l.append((4, 4))
        dl.append((4, 4))
        self.assertTrue(dl == l)

    def test_len(self):
        ta = (1, 2, 3)
        l = zip(ta, ta)
        dl = IntDisplayList(l)
        self.assertTrue(len(dl) == len(l))

    def test_keys(self):
        ta = (1, 2, 3)
        l = zip(ta, ta)
        dl = IntDisplayList(l)
        self.assertTrue(tuple(dl.keys()) == ta)

    def test_values(self):
        ta = (1, 2, 3)
        l = zip(ta, ta)
        dl = IntDisplayList(l)
        self.assertTrue(tuple(dl.values()) == ta)

    def test_items(self):
        ta = (1, 2, 3)
        l = zip(ta, ta)
        dl = IntDisplayList(l)
        self.assertTrue(dl.items() == tuple(l))

    def test_repr(self):
        ta = (1, 2, 3)
        l = zip(ta, ta)
        dl = IntDisplayList(l)
        self.assertTrue(repr(dl).find(str(l)))

    def test_str(self):
        ta = (1, 2, 3)
        l = zip(ta, ta)
        dl = IntDisplayList(l)
        self.assertTrue(str(dl) == str(l))

    def test_call(self):
        ta = (1, 2, 3)
        l = zip(ta, ta)
        dl = IntDisplayList(l)
        self.assertTrue(dl == dl)
        self.assertTrue(dl() == dl())
        self.assertTrue(dl[:] == l)
        self.assertTrue(dl()[:] == l)

    def test_sort(self):
        a = ((1, 'a',), (2, 'b'), (3, 'c'))
        b = ((10, 'Z',), (9, 'Y'), (8, 'X'))
        c = ((1, 'Z',), (3, 'Y'), (2, 'X'))
        dla = IntDisplayList(a)
        dlb = IntDisplayList(b)
        dlc = IntDisplayList(c)

        assert dla.values() == ['a', 'b', 'c']
        dlb_s = dlb.sortedByValue()
        assert dlb_s.values() == ['X', 'Y', 'Z']
        dlc_s = dlc.sortedByKey()
        assert dlc_s.values() == ['Z', 'X', 'Y']

    def test_getValue(self):
        a = ((1, 'A',), (2, '\xc3\xab'), (3, u'xeb'), (4, 42))
        dla = IntDisplayList(a)
        self.assertEqual(dla.getValue(1), 'A')
        self.assertEqual(dla.getValue(2), '\xc3\xab')
        self.assertEqual(dla.getValue(3), u'xeb')
        self.assertEqual(dla.getValue(4), 42)
        self.assertEqual(dla.getValue(5), None)
        self.assertEqual(dla.getValue(5, 'default'), 'default')


class VocabularyTest(ATTestCase):

    def test_getValue(self):
        a = (('a', 'A',), ('b', '\xc3\xab'), ('c', u'xeb'), ('d', 42))
        dla = DisplayList(a)
        va = Vocabulary(dla, instance=None, i18n_domain=None)
        self.assertEqual(va.getValue('a'), 'A')
        self.assertEqual(va.getValue('b'), '\xc3\xab')
        self.assertEqual(va.getValue('c'), u'xeb')
        self.assertEqual(va.getValue('d'), 42)
        self.assertEqual(va.getValue('e'), None)
        self.assertEqual(va.getValue('e', 'default'), 'default')

        b = (('A', 'a',), ('\xc3\xab', 'b'), (u'xeb', 'c'))
        dlb = DisplayList(b)
        vb = Vocabulary(dlb, instance=None, i18n_domain=None)
        self.assertEqual(vb.getValue('A'), 'a')
        self.assertEqual(vb.getValue('\xc3\xab'), 'b')
        self.assertEqual(vb.getValue(u'xeb'), 'c')
        self.assertEqual(vb.getValue('e'), None)
        self.assertEqual(vb.getValue('e', 'default'), 'default')

        c = ((1, 'A',), (2, '\xc3\xab'), (3, u'xeb'), (4, 42))
        dlc = IntDisplayList(c)
        vb = Vocabulary(dlc, instance=None, i18n_domain=None)
        self.assertEqual(dlc.getValue(1), 'A')
        self.assertEqual(dlc.getValue(2), '\xc3\xab')
        self.assertEqual(dlc.getValue(3), u'xeb')
        self.assertEqual(dlc.getValue(4), 42)
        self.assertEqual(dlc.getValue(5), None)
        self.assertEqual(dlc.getValue(5, 'default'), 'default')

    def test_translating_getValue(self):
        # We use the same base displaylists as above (hopefully), but
        # now we pass an instance and an i18n_domain.  The instance is
        # expected to be an Archetypes object, but currently it only
        # needs to have a True boolean value.
        a = (('a', 'A',), ('b', '\xc3\xab'), ('c', u'xeb'), ('d', 42))
        dla = DisplayList(a)
        va = Vocabulary(dla, instance=object(), i18n_domain='plone')
        self.assertEqual(va.getValue('a'), 'A')
        self.assertEqual(va.getValue('b'), '\xc3\xab'.decode('utf-8'))
        self.assertEqual(va.getValue('c'), u'xeb')
        self.assertEqual(va.getValue('d'), 42)
        self.assertEqual(va.getValue('e'), None)
        self.assertEqual(va.getValue('e', 'default'), 'default')

        b = (('A', 'a',), ('\xc3\xab', 'b'), (u'xeb', 'c'))
        dlb = DisplayList(b)
        vb = Vocabulary(dlb, instance=object(), i18n_domain='plone')
        self.assertEqual(vb.getValue('A'), 'a')
        self.assertEqual(vb.getValue('\xc3\xab'), 'b')
        self.assertEqual(vb.getValue(u'xeb'), 'c')
        self.assertEqual(vb.getValue('e'), None)
        self.assertEqual(vb.getValue('e', 'default'), 'default')

        c = ((1, 'A',), (2, '\xc3\xab'), (3, u'xeb'), (4, 42))
        dlc = IntDisplayList(c)
        vb = Vocabulary(dlc, instance=object(), i18n_domain='plone')
        self.assertEqual(dlc.getValue(1), 'A')
        self.assertEqual(dlc.getValue(2), '\xc3\xab')
        self.assertEqual(dlc.getValue(3), u'xeb')
        self.assertEqual(dlc.getValue(4), 42)
        self.assertEqual(dlc.getValue(5), None)
        self.assertEqual(dlc.getValue(5, 'default'), 'default')


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(VocabularyTest))
    suite.addTest(makeSuite(DisplayListTest))
    suite.addTest(makeSuite(IntDisplayListTest))
    suite.addTest(makeSuite(UidGeneratorTest))
    return suite



from zope.interface import Attribute, Interface


class IArchetypeTool(Interface):
    """This tool manages various kinds of behaviour for Archetype based
    content types. """

    id = Attribute('id', 'Must be set to "archetype_tool"')

    def listRegisteredTypes(inProject=False):
        """Return the list of sorted types.
        """

    def setCatalogsByType(meta_type, catalogList):
        """ associate catalogList with meta_type. (unfortunally not portal_type).

            catalogList is a list of strings with the ids of the catalogs.
            Each catalog is has to be a tool, means unique in site root.
        """

    def getCatalogsByType(meta_type):
        """Return the catalog objects assoicated with a given type.
        """

    def getCatalogsInSite(self):
        """Return a list of ids for objects implementing ZCatalog.
        """



################################################################################
#
# Copyright (c) 2002-2005, Benjamin Saller <bcsaller@ideasuite.com>, and
#                              the respective authors. All rights reserved.
# For a list of Archetypes contributors see docs/CREDITS.txt.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the author nor the names of its contributors may be used
#   to endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
################################################################################

from unittest import TestSuite, makeSuite

from Acquisition import aq_base
import transaction

from Products.Archetypes.tests.atsitetestcase import ATSiteTestCase
from Products.Archetypes.tests.utils import makeContent

from Products.Archetypes.config import REFERENCE_CATALOG, UID_CATALOG, UUID_ATTR

from Products.Archetypes.atapi import DisplayList
from plone.uuid.interfaces import IUUIDAware, IUUID


class BaseReferenceableTests(ATSiteTestCase):

    FOLDER_TYPE = None

    def verifyBrains(self):
        uc = getattr(self.portal, UID_CATALOG)
        rc = getattr(self.portal, REFERENCE_CATALOG)

        #Verify all UIDs resolve
        uids = uc.uniqueValuesFor('UID')
        brains = uc(dict(UID=uids))

        uobjects = [b.getObject() for b in brains]
        self.assertFalse(None in uobjects, """bad uid resolution""")

        #Verify all references resolve
        uids = rc.uniqueValuesFor('UID')
        brains = rc(dict(UID=uids))

        robjects = [b.getObject() for b in brains]
        self.assertFalse(None in robjects, """bad ref catalog resolution""")
        return uobjects, robjects

    def test_hasUID(self):
        doc = makeContent(self.folder,
                          portal_type='DDocument',
                          title='Foo')

        self.assertTrue(hasattr(aq_base(doc), UUID_ATTR))
        self.assertTrue(getattr(aq_base(doc), UUID_ATTR, None))

    def test_uuid(self):
        doc = makeContent(self.folder,
                          portal_type='DDocument',
                          title='Foo')

        self.assertTrue(IUUIDAware.providedBy(doc))
        uuid = IUUID(doc, None)
        self.assertTrue(uuid == doc.UID())

    def test_renamedontchangeUID(self):
        catalog = self.portal.uid_catalog

        obj_id = 'demodoc'
        new_id = 'new_demodoc'
        doc = makeContent(self.folder,
                          portal_type='DDocument',
                          title='Foo',
                          id=obj_id)

        UID = doc.UID()
        # This test made an assumption about other UIDs in the system
        # that are wrong with things like ATCT
        self.assertTrue(UID in catalog.uniqueValuesFor('UID'))
        # ensure object has a _p_jar
        transaction.savepoint(optimistic=True)
        self.folder.manage_renameObject(id=obj_id, new_id=new_id)
        doc = getattr(self.folder, new_id)
        self.assertTrue(UID in catalog.uniqueValuesFor('UID'))
        self.assertEquals(doc.UID(), UID)

    def test_renameKeepsReferences(self):
        container = makeContent(self.folder,
                                portal_type=self.FOLDER_TYPE,
                                title='Spam',
                                id='container')

        obj1 = makeContent(self.folder.container,
                           portal_type='SimpleType',
                           title='Eggs',
                           id='obj1')
        obj2 = makeContent(self.folder.container,
                           portal_type='SimpleType',
                           title='Foo',
                           id='obj2')

        obj1.addReference(obj2)

        self.verifyBrains()
        transaction.savepoint(optimistic=True)
        obj1.setId('foo')
        transaction.savepoint(optimistic=True)

        self.assertEquals(obj2.getBRefs(), [obj1])
        self.assertEquals(obj1.getRefs(), [obj2])

        self.verifyBrains()
        transaction.savepoint(optimistic=True)
        obj2.setId('bar')
        transaction.savepoint(optimistic=True)

        self.assertEquals(obj2.getBRefs(), [obj1])
        self.assertEquals(obj1.getRefs(), [obj2])

        self.verifyBrains()

    def test_renamecontainerKeepsReferences(self):
        # test for #956677: renaming the container causes contained objects
        #                   to lose their refs
        container = makeContent(self.folder,
                                portal_type=self.FOLDER_TYPE,
                                title='Spam',
                                id='container')
        obj1 = makeContent(self.folder.container,
                           portal_type='SimpleType',
                           title='Eggs',
                           id='obj1')
        obj2 = makeContent(self.folder,
                           portal_type='SimpleType',
                           title='Foo',
                           id='obj2')

        obj1.addReference(obj2)

        a, b = self.verifyBrains()
        transaction.savepoint(optimistic=True)

        self.assertEquals(obj2.getBRefs(), [obj1])
        self.assertEquals(obj1.getRefs(), [obj2])

        self.folder.manage_renameObject(id='container',
                                        new_id='cont4iner')
        c, d = self.verifyBrains()

        obj1 = self.folder.cont4iner.obj1
        obj2 = self.folder.cont4iner.obj2

        self.assertEquals(obj2.getBRefs(), [obj1])
        self.assertEquals(obj1.getRefs(), [obj2])

    def test_renamecontainerKeepsReferences2(self):
        # test for [ 1013363 ] References break on folder rename
        folderA = makeContent(self.folder,
                                portal_type=self.FOLDER_TYPE,
                                title='Spam',
                                id='folderA')
        objA = makeContent(self.folder.folderA,
                           portal_type='SimpleType',
                           title='Eggs',
                           id='objA')

        folderB = makeContent(self.folder,
                                portal_type=self.FOLDER_TYPE,
                                title='Spam',
                                id='folderB')
        objB = makeContent(self.folder.folderB,
                           portal_type='SimpleType',
                           title='Eggs',
                           id='objB')

        objA.addReference(objB)

        a, b = self.verifyBrains()
        transaction.savepoint(optimistic=True)

        self.assertEquals(objB.getBRefs(), [objA])
        self.assertEquals(objA.getRefs(), [objB])

        # now rename folder B and see if objA still points to objB
        self.folder.manage_renameObject(id='folderB',
                                        new_id='folderC')
        c, d = self.verifyBrains()

        objB = self.folder.folderC.objB

        # check references
        self.assertEquals(objB.getBRefs(), [objA])
        self.assertEquals(objA.getRefs(), [objB])

    def test_UIDclash(self):
        catalog = getattr(self.portal, UID_CATALOG)

        obj_id = 'demodoc'
        new_id = 'new_demodoc'
        doc = makeContent(self.folder,
                          portal_type='DDocument',
                          title='Foo',
                          id=obj_id)

        UID = doc.UID()
        # ensure object has a _p_jar
        transaction.savepoint(optimistic=True)
        self.folder.manage_renameObject(id=obj_id, new_id=new_id)

        #now, make a new one with the same ID and check it gets a different UID
        doc2 = makeContent(self.folder,
                           portal_type='DDocument',
                           title='Foo',
                           id=obj_id)

        UID2 = doc2.UID()
        self.assertFalse(UID == UID2)
        uniq = catalog.uniqueValuesFor('UID')
        self.assertTrue(UID in uniq, (UID, uniq))
        self.assertTrue(UID2 in uniq, (UID, uniq))

    def test_setUID_keeps_relationships(self):
        obj_id = 'demodoc'
        known_id = 'known_doc'
        owned_id = 'owned_doc'

        a = makeContent(self.folder, portal_type='DDocument',
                        title='Foo', id=obj_id)
        b = makeContent(self.folder, portal_type='DDocument',
                        title='Foo', id=known_id)
        c = makeContent(self.folder, portal_type='DDocument',
                        title='Foo', id=owned_id)

        #Two made up kinda refs
        a.addReference(b, "KnowsAbout")
        b.addReference(a, "KnowsAbout")
        a.addReference(c, "Owns")

        refs = a.getRefs()
        self.assertTrue(b in refs, (b, refs))
        self.assertTrue(c in refs, (c, refs))
        self.assertEquals(a.getRefs('KnowsAbout'), [b])
        self.assertEquals(b.getRefs('KnowsAbout'), [a])
        self.assertEquals(a.getRefs('Owns'), [c])
        self.assertEquals(c.getBRefs('Owns'), [a])

        old_uid = a.UID()

        # Check existing forward refs
        fw_refs = a.getReferenceImpl()
        old_refs = []
        [old_refs.append(o.sourceUID) for o in fw_refs
         if not o.sourceUID in old_refs]
        self.assertEquals(len(old_refs), 1)
        self.assertEquals(old_refs[0], old_uid)

        # Check existing backward refs
        fw_refs = a.getBackReferenceImpl()
        old_refs = []
        [old_refs.append(o.targetUID) for o in fw_refs
         if not o.targetUID in old_refs]
        self.assertEquals(len(old_refs), 1)
        self.assertEquals(old_refs[0], old_uid)

        new_uid = '9x9x9x9x9x9x9x9x9x9x9x9x9x9x9x9x9'
        a._setUID(new_uid)
        self.assertEquals(a.UID(), new_uid)

        # Check existing forward refs got reassigned
        fw_refs = a.getReferenceImpl()
        new_refs = []
        [new_refs.append(o.sourceUID) for o in fw_refs
         if not o.sourceUID in new_refs]
        self.assertEquals(len(new_refs), 1)
        self.assertEquals(new_refs[0], new_uid)

        # Check existing backward refs got reassigned
        fw_refs = a.getBackReferenceImpl()
        new_refs = []
        [new_refs.append(o.targetUID) for o in fw_refs
         if not o.targetUID in new_refs]
        self.assertEquals(len(new_refs), 1)
        self.assertEquals(new_refs[0], new_uid)

        refs = a.getRefs()
        self.assertTrue(b in refs, (b, refs))
        self.assertTrue(c in refs, (c, refs))
        self.assertEquals(a.getRefs('KnowsAbout'), [b])
        self.assertEquals(b.getRefs('KnowsAbout'), [a])
        self.assertEquals(a.getRefs('Owns'), [c])
        self.assertEquals(c.getBRefs('Owns'), [a])

        self.verifyBrains()

    def test_relationships(self):

        obj_id = 'demodoc'
        known_id = 'known_doc'
        owned_id = 'owned_doc'
        other_id = 'other_doc'

        a = makeContent(self.folder, portal_type='DDocument',
                        title='Foo', id=obj_id)
        b = makeContent(self.folder, portal_type='DDocument',
                        title='Foo', id=known_id)
        c = makeContent(self.folder, portal_type='DDocument',
                        title='Foo', id=owned_id)

        #Two made up kinda refs
        a.addReference(b, "KnowsAbout")
        a.addReference(c, "Owns")

        refs = a.getRefs()
        self.assertTrue(b in refs, (b, refs))
        self.assertTrue(c in refs, (c, refs))
        self.assertEquals(a.getRefs('Owns'), [c])
        self.assertEquals(c.getBRefs('Owns'), [a])
        rels = a.getRelationships()
        self.assertTrue("KnowsAbout" in rels, ("KnowsAbout", rels))
        self.assertTrue("Owns" in rels, ("Owns", rels))

        a.deleteReference(c, "Owns")
        self.assertEquals(a.getRefs(), [b])
        self.assertEquals(c.getBRefs(), [])

        #test querying references using the targetObject parameter
        d = makeContent(self.folder, portal_type='DDocument',
                        title='Foo', id=other_id)

        a.addReference(d, 'Owns')
        a.addReference(d, 'KnowsAbout')

        self.assertEqual(len(a.getReferenceImpl()), 3)
        #get only refs to d
        self.assertEqual(len(a.getReferenceImpl(targetObject=d)), 2)

    def test_back_relationships(self):

        account_id = 'caixa'
        invoice_id = 'fatura'
        payment_id = 'entrada'
        future_payment_id = 'cta_receber'
        payment2_id = 'quitacao'

        account = makeContent(self.folder, portal_type='DDocument',
                              title='Account', id=account_id)
        invoice = makeContent(self.folder, portal_type='DDocument',
                              title='Invoice', id=invoice_id)
        payment = makeContent(self.folder, portal_type='DDocument',
                              title='Payment', id=payment_id)
        future_payment = makeContent(self.folder, portal_type='DDocument',
                                     title='Future Payment',
                                     id=future_payment_id)
        payment2 = makeContent(self.folder, portal_type='DDocument',
                               title='Payment 2', id=payment2_id)

        invoice.addReference(payment, "Owns")
        invoice.addReference(future_payment, "Owns")
        future_payment.addReference(payment2, "Owns")
        payment.addReference(account, "From")
        payment2.addReference(account, "From")

        brels = account.getBRelationships()
        self.assertEquals(brels, ['From'])
        brefs = account.getBRefs('From')
        # The order is not defined, which can lead to spurious test
        # failures, but we do not care about the order.
        self.assertEquals(len(brefs), 2)
        self.assertTrue(payment in brefs)
        self.assertTrue(payment2 in brefs)

        brels = payment.getBRelationships()
        self.assertEquals(brels, ['Owns'])
        brefs = payment.getBRefs('Owns')
        self.assertEquals(brefs, [invoice])

        brels = payment2.getBRelationships()
        self.assertEquals(brels, ['Owns'])
        brefs = payment2.getBRefs('Owns')
        self.assertEquals(brefs, [future_payment])

        invoice.deleteReference(payment, "Owns")

        self.assertEquals(invoice.getRefs(), [future_payment])
        self.assertEquals(payment.getBRefs(), [])

    def test_singleReference(self):
        # If an object is referenced don't record its reference again
        a = makeContent(self.folder, portal_type='DDocument', title='Foo', id='a')
        b = makeContent(self.folder, portal_type='DDocument', title='Foo', id='b')

        #Add the same ref twice
        a.addReference(b, "KnowsAbout")
        a.addReference(b, "KnowsAbout")

        self.assertEquals(len(a.getRefs('KnowsAbout')),  1)

        #In this case its a different relationship
        a.addReference(b, 'Flogs')
        self.assertEquals(len(a.getRefs('KnowsAbout')), 1)
        self.assertEquals(len(a.getRefs()), 2)

    def test_multipleReferences(self):
        # If you provide updateReferences=False to addReference, it
        # will add, not replace the reference
        a = makeContent(self.folder, portal_type='DDocument', title='Foo', id='a')
        b = makeContent(self.folder, portal_type='DDocument', title='Foo', id='b')

        #Add the same ref twice
        a.addReference(b, "KnowsAbout", updateReferences=False)
        a.addReference(b, "KnowsAbout", updateReferences=False)

        self.assertEquals(len(a.getRefs('KnowsAbout')),  2)

        #In this case its a different relationship
        a.addReference(b, 'Flogs')
        self.assertEquals(len(a.getRefs('KnowsAbout')), 2)
        self.assertEquals(len(a.getRefs()), 3)

    def test_UIDunderContainment(self):
        # If an object is referenced don't record its reference again
        at = self.portal.archetype_tool

        folder = makeContent(self.folder, portal_type=self.FOLDER_TYPE,
                             title='Foo', id='folder')
        nonRef = makeContent(folder, portal_type='Document',
                             title='Foo', id='nonRef')

        fuid = folder.UID()
        nuid = nonRef.UID()
        # We expect this to break, an aq_explicit would fix it but
        # we can't change the calling convention
        # XXX: but proxy index could
        # XXX: assert fuid != nuid

    def test_hasRelationship(self):
        a = makeContent(self.folder, portal_type='DDocument', title='Foo', id='a')
        b = makeContent(self.folder, portal_type='DDocument', title='Foo', id='b')
        c = makeContent(self.folder, portal_type='DDocument', title='Foo', id='c')

        # Two made up kinda refs
        a.addReference(b, "KnowsAbout")

        self.assertEquals(a.hasRelationshipTo(b), 1)
        self.assertEquals(a.hasRelationshipTo(b, "KnowsAbout"), 1)
        self.assertEquals(a.hasRelationshipTo(b, "Foo"), 0)
        self.assertEquals(a.hasRelationshipTo(c), 0)
        self.assertEquals(a.hasRelationshipTo(c, "KnowsAbout"), 0)

        # XXX HasRelationshipFrom  || ( 1 for ref 2 for bref?)

    def test_folderishDeleteCleanup(self):
        self.folder.invokeFactory(type_name="Folder", id="reftest")
        folder = getattr(self.folder, "reftest")

        a = makeContent(folder, portal_type='DDocument', title='Foo', id='a')
        b = makeContent(folder, portal_type='DDocument', title='Bar', id='b')
        a.addReference(b, "KnowsAbout")

        # Again, lets assert the sanity of the UID and Ref Catalogs
        uc = self.portal.uid_catalog
        rc = self.portal.reference_catalog

        uids = uc.uniqueValuesFor('UID')
        self.assertTrue(a.UID() in uids, (a.UID(), uids))
        self.assertTrue(b.UID() in uids, (b.UID(), uids))

        uids = rc.uniqueValuesFor('UID')
        refs = rc(dict(UID=uids))
        self.assertEquals(len(refs), 1)
        ref = refs[0].getObject()
        self.assertEquals(ref.targetUID, b.UID())
        self.assertEquals(ref.sourceUID, a.UID())

        # Now Kill the folder and make sure it all went away
        self.folder._delObject("reftest")
        self.verifyBrains()

        uids = rc.uniqueValuesFor('UID')
        self.assertEquals(len(uids), 0)

    def test_reindexUIDCatalog(self):
        catalog = self.portal.uid_catalog

        doc = makeContent(self.folder,
                          portal_type='DDocument',
                          id='demodoc')
        doc.update(title="sometitle")
        brain = catalog(dict(UID=doc.UID()))[0]
        self.assertEquals(brain.Title, doc.Title())

    def test_referenceReference(self):
        # Reference a reference object for fun (no, its like RDFs
        # metamodel)
        rc = self.portal.reference_catalog

        a = makeContent(self.folder, portal_type='DDocument', title='Foo', id='a')
        b = makeContent(self.folder, portal_type='DDocument', title='Foo', id='b')
        c = makeContent(self.folder, portal_type='DDocument', title='Foo', id='c')
        a.addReference(b)

        ref = a._getReferenceAnnotations().objectValues()[0]
        c.addReference(ref)
        ref.addReference(c)
        self.verifyBrains()

    def test_referenceFieldVocab(self):
        dummy = makeContent(self.folder, portal_type="Refnode", id="dummy")
        test123 = makeContent(self.folder, portal_type="Refnode",
                              id="Test123")
        test124 = makeContent(self.folder, portal_type="Refnode",
                              id="Test124")
        test125 = makeContent(self.folder, portal_type="Refnode",
                              id="Test125")

        field = dummy.Schema()['adds']

        expected = DisplayList([
            (test123.UID(), test123.getId()),
            (test124.UID(), test124.getId()),
            (test125.UID(), test125.getId()),
            (dummy.UID(), dummy.getId()),
            ])

        got = field.Vocabulary(dummy)
        self.assertEquals(got, expected)

        # We should have the option of nothing
        field = field.copy()
        field.required = 0
        field.multiValued = 0

        expected = DisplayList([
            (test123.UID(), test123.getId()),
            (test124.UID(), test124.getId()),
            (test125.UID(), test125.getId()),
            (dummy.UID(), dummy.getId()),
            ('', u'label_no_reference'),
            ])
        self.assertEquals(field.Vocabulary(dummy), expected)

        field = field.copy()
        field.vocabulary_display_path_bound = 1
        expected = DisplayList([
            (test123.UID(), test123.getId()),
            (test124.UID(), test124.getId()),
            (test125.UID(), test125.getId()),
            (dummy.UID(), dummy.getId()),
            ('', u'label_no_reference'),
            ])
        self.assertNotEqual(field.Vocabulary(dummy), expected)
        field.vocabulary_display_path_bound = -1
        self.assertEquals(field.Vocabulary(dummy), expected)

    def test_noReferenceAfterDelete(self):
        # Deleting target should delete reference
        # added by GL
        a = makeContent(self.folder, portal_type='DDocument', id='a')
        b = makeContent(self.folder, portal_type='DDocument', id='b')
        a.addReference(b)
        self.folder._delObject('b')

        self.assertEqual(a.getRefs(), [])

    def test_noBackReferenceAfterDelete(self):
        # Deleting source should delete back reference
        # added by GL
        a = makeContent(self.folder, portal_type='DDocument', id='a')
        b = makeContent(self.folder, portal_type='DDocument', id='b')
        a.addReference(b)
        self.folder._delObject('a')

        self.assertEqual(b.getBRefs(), [])

    def test_copyKeepsReferences(self):
        # when copied a pasted object should NOT lose all references
        # if keepReferencesOnCopy is set
        # added by DaftDog (for plone tracker issue  #5180)
        org_folder = makeContent(self.folder,
                                 portal_type=self.FOLDER_TYPE,
                                 title='Origin folder',
                                 id='org_folder')
        dst_folder = makeContent(self.folder,
                                 portal_type=self.FOLDER_TYPE,
                                 title='Destination folder',
                                 id='dst_folder')
        a = makeContent(org_folder, portal_type='DDocument', id='a')
        b = makeContent(org_folder, portal_type='DDocument', id='b')
        related_field = a.getField('related')
        related_field.set(a, b.UID())

        self.assertEqual(b.getBRefs(), [a])
        self.assertEqual(a.getRefs(), [b])

        cb = org_folder.manage_copyObjects(ids=['a'])
        dst_folder.manage_pasteObjects(cb_copy_data=cb)
        copy_a = getattr(dst_folder, 'a')

        # The copy should get a new UID
        a_uid = a.UID()
        ca_uid = copy_a.UID()
        self.assertFalse(a_uid == ca_uid, (a_uid, ca_uid))

        # The copy should have the same references
        self.assertEqual(a.getRefs(), copy_a.getRefs())
        self.assertTrue(copy_a in b.getBRefs())

        # Original object should keep references
        self.assertEqual(a.getRefs(), [b])
        # Original non-copied object should point to both the original and the copied object
        self.assertEqual(b.getBRefs(), [a, copy_a])

    def test_copyPasteSupport(self):
        # copy/paste behaviour test
        # in another folder, pasted object should lose all references
        # added by GL (for bug #985393)
        org_folder = makeContent(self.folder,
                                 portal_type=self.FOLDER_TYPE,
                                 title='Origin folder',
                                 id='org_folder')
        dst_folder = makeContent(self.folder,
                                 portal_type=self.FOLDER_TYPE,
                                 title='Destination folder',
                                 id='dst_folder')
        a = makeContent(org_folder, portal_type='DDocument', id='a')
        b = makeContent(org_folder, portal_type='DDocument', id='b')
        a.addReference(b)

        self.assertEqual(b.getBRefs(), [a])
        self.assertEqual(a.getRefs(), [b])

        cb = org_folder.manage_copyObjects(ids=['a'])
        dst_folder.manage_pasteObjects(cb_copy_data=cb)
        copy_a = getattr(dst_folder, 'a')

        # The copy should get a new UID
        a_uid = a.UID()
        ca_uid = copy_a.UID()
        self.assertFalse(a_uid == ca_uid, (a_uid, ca_uid))

        # The copy shouldn't have references
        self.assertEqual(copy_a.getRefs(), [])
        self.assertFalse(copy_a in b.getBRefs())

        # Original object should keep references
        self.assertEqual(a.getRefs(), [b])
        self.assertEqual(b.getBRefs(), [a])

    def test_cutPasteSupport(self):
        # cut/paste behaviour test
        # in another folder, pasted object should keep the references
        # added by GL (for bug #985393)
        org_folder = makeContent(self.folder,
                                 portal_type=self.FOLDER_TYPE,
                                 title='Origin folder',
                                 id='org_folder')
        dst_folder = makeContent(self.folder,
                                 portal_type=self.FOLDER_TYPE,
                                 title='Destination folder',
                                 id='dst_folder')
        a = makeContent(org_folder, portal_type='DDocument', id='a')
        b = makeContent(org_folder, portal_type='DDocument', id='b')
        a.addReference(b)
        transaction.savepoint(optimistic=True)
        cb = org_folder.manage_cutObjects(ids=['a'])
        dst_folder.manage_pasteObjects(cb_copy_data=cb)
        copy_a = getattr(dst_folder, 'a')

        self.assertEqual(copy_a.getRefs(), [b])
        self.assertEqual(b.getBRefs(), [copy_a])

    def test_folderCopyPasteSupport(self):
        # copy/paste behaviour test
        # sub-objects of copy/pasted folders should lose all references,
        # and duplicate refs should not be created on the original object.
        org_folder = makeContent(self.folder,
                                 portal_type=self.FOLDER_TYPE,
                                 title='Origin folder',
                                 id='org_folder')
        dst_folder = makeContent(self.folder,
                                 portal_type=self.FOLDER_TYPE,
                                 title='Destination folder',
                                 id='dst_folder')
        my_folder = makeContent(org_folder, portal_type=self.FOLDER_TYPE,
                                                             id='my_folder')
        a = makeContent(my_folder, portal_type='DDocument', id='a')
        b = makeContent(my_folder, portal_type='DDocument', id='b')
        a.addReference(b)

        self.assertEqual(b.getBRefs(), [a])
        self.assertEqual(a.getRefs(), [b])

        cb = org_folder.manage_copyObjects(ids=['my_folder'])
        dst_folder.manage_pasteObjects(cb_copy_data=cb)
        copy_folder = getattr(dst_folder, 'my_folder')
        copy_a = getattr(copy_folder, 'a')

        # The copy should get a new UID
        a_uid = a.UID()
        ca_uid = copy_a.UID()
        self.assertFalse(a_uid == ca_uid, (a_uid, ca_uid))

        # The copy shouldn't have references
        self.assertEqual(copy_a.getRefs(), [])
        self.assertFalse(copy_a in b.getBRefs())

        #The copy's uid should have changed
        self.assertFalse(ca_uid == a_uid)

        # Original object should keep references
        self.assertEqual(a.getRefs(), [b])
        self.assertEqual(b.getBRefs(), [a])

    def test_folderCutPasteSupport(self):
        # copy/paste behaviour test
        # sub-objects of copy/pasted folders should lose all references,
        # and duplicate refs should not be created on the original object.
        org_folder = makeContent(self.folder,
                                 portal_type=self.FOLDER_TYPE,
                                 title='Origin folder',
                                 id='org_folder')
        dst_folder = makeContent(self.folder,
                                 portal_type=self.FOLDER_TYPE,
                                 title='Destination folder',
                                 id='dst_folder')
        my_folder = makeContent(org_folder, portal_type=self.FOLDER_TYPE,
                                                             id='my_folder')
        a = makeContent(my_folder, portal_type='DDocument', id='a')
        b = makeContent(my_folder, portal_type='DDocument', id='b')
        a.addReference(b)

        self.assertEqual(b.getBRefs(), [a])
        self.assertEqual(a.getRefs(), [b])
        a_uid = a.UID()

        transaction.savepoint(optimistic=True)
        cb = org_folder.manage_cutObjects(ids=['my_folder'])
        dst_folder.manage_pasteObjects(cb_copy_data=cb)
        copy_folder = getattr(dst_folder, 'my_folder')
        copy_a = getattr(copy_folder, 'a')
        copy_b = getattr(copy_folder, 'b')
        ca_uid = copy_a.UID()

        # The copy shouldn't have references
        self.assertEqual(copy_a.getRefs(), [copy_b])
        self.assertEqual(copy_b.getBRefs(), [copy_a])

        #The copy's uid should have changed
        self.assertTrue(ca_uid == a_uid, (a_uid, ca_uid))


class SimpleFolderReferenceableTests(BaseReferenceableTests):
    FOLDER_TYPE = 'SimpleFolder'


class SimpleBTreeFolderReferenceableTests(BaseReferenceableTests):
    FOLDER_TYPE = 'SimpleBTreeFolder'


def test_suite():
    suite = TestSuite()
    suite.addTest(makeSuite(SimpleFolderReferenceableTests))
    suite.addTest(makeSuite(SimpleBTreeFolderReferenceableTests))
    return suite



import os

from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from OFS.Folder import Folder
from Products.CMFCore.utils import UniqueObject
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

_www = os.path.join(os.path.dirname(__file__), 'www')


class ArchTTWTool(UniqueObject, Folder):
    """ Archetypes TTW Tool """

    id = 'archetypes_ttw_tool'
    meta_type = 'Archetypes TTW Tool'

    security = ClassSecurityInfo()
    meta_types = all_meta_types = (())

    manage_options = (
        (Folder.manage_options[0],) +
        (
        {'label': 'Introspect',
         'action': 'manage_debugForm',
         },

        )
        + Folder.manage_options[1:]
        )

    fields_template = PageTemplateFile('fields_xml', _www)
    widgets_template = PageTemplateFile('widgets_xml', _www)
    storages_template = PageTemplateFile('storages_xml', _www)
    validators_template = PageTemplateFile('validators_xml', _www)
    types_template = PageTemplateFile('types_xml', _www)
    type_template = PageTemplateFile('type_xml', _www)
    registry_template = PageTemplateFile('registry_xml', _www)

    def fields(self):
        from Registry import availableFields
        fields = [v for k, v in availableFields()]
        return fields

    security.declarePublic('fields_xml')
    def fields_xml(self):
        """ Return XML representation of the field registry """
        fields = self.fields()
        return self.fields_template(fields=fields)

    def widgets(self):
        from Registry import availableWidgets
        widgets = [v for k, v in availableWidgets()]
        return widgets

    security.declarePublic('widgets_xml')
    def widgets_xml(self):
        """ Return XML representation of the widget registry """
        widgets = self.widgets()
        return self.widgets_template(widgets=widgets)

    def storages(self):
        from Registry import availableStorages
        storages = [v for k, v in availableStorages()]
        return storages

    security.declarePublic('storages_xml')
    def storages_xml(self):
        """ Return XML representation of the storage registry """
        storages = self.storages()
        return self.storages_template(storages=storages)

    def validators(self):
        from Registry import availableValidators
        validators = [v for k, v in availableValidators()]
        return validators

    security.declarePublic('validators_xml')
    def validators_xml(self):
        """ Return XML representation of the validators registry """
        validators = self.validators()
        return self.validators_template(validators=validators)

    def types(self):
        from Registry import availableTypes
        types = [v for k, v in availableTypes()]
        return types

    security.declarePublic('types_xml')
    def types_xml(self):
        """ Return XML representation of the types registry """
        types = self.types()
        return self.types_template(types=types)

    security.declarePublic('registry_xml')
    def registry_xml(self):
        """ Return XML representation of the wholeregistry """
        options = {}
        options['fields'] = self.fields()
        options['widgets'] = self.widgets()
        options['storages'] = self.storages()
        options['validators'] = self.validators()
        options['types'] = self.types()
        return self.registry_template(**options)

    security.declarePublic('type_xml')
    def type_xml(self):
        """ Return XML representation of one type from the registry """
        type = self.REQUEST.get('type', '')
        if not type:
            raise ValueError, 'Type is not valid'
        from Products.Archetypes.Registry import typeDescriptionRegistry
        type = typeDescriptionRegistry[type]
        return self.type_template(type=type)


InitializeClass(ArchTTWTool)



from Acquisition import aq_inner
from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName
import json

SKIP_VALIDATION_FIELDTYPES = ('image', 'file')


class InlineValidationView(BrowserView):

    def __call__(self, uid, fname, value):
        '''Validate a given field. Return any error messages.
        '''
        res = {'errmsg': ''}

        rc = getToolByName(aq_inner(self.context), 'reference_catalog')
        instance = rc.lookupObject(uid)
        # make sure this works for portal_factory items
        if instance is None:
            instance = self.context

        field = instance.getField(fname)
        if field and field.type not in SKIP_VALIDATION_FIELDTYPES:
            error = field.validate(value, instance, {}, REQUEST=self.request)
            if isinstance(error, str):
                error = error.decode('utf', 'replace')
            res['errmsg'] = error or ''

        self.request.response.setHeader('Content-Type', 'application/json')
        return json.dumps(res)



"""Event definitions
"""

from zope.interface import implements

from zope.component.interfaces import ObjectEvent
from zope.lifecycleevent import ObjectModifiedEvent

from Products.Archetypes.interfaces import IObjectInitializedEvent
from Products.Archetypes.interfaces import IWebDAVObjectInitializedEvent
from Products.Archetypes.interfaces import IObjectEditedEvent
from Products.Archetypes.interfaces import IWebDAVObjectEditedEvent
from Products.Archetypes.interfaces import IEditBegunEvent
from Products.Archetypes.interfaces import IEditCancelledEvent

# Modification

class ObjectInitializedEvent(ObjectModifiedEvent):
    """An object is being initialised, i.e. populated for the first time
    """
    implements(IObjectInitializedEvent)

class WebDAVObjectInitializedEvent(ObjectInitializedEvent):
    """An object is being initialised via WebDAV
    """
    implements(IWebDAVObjectInitializedEvent)

class ObjectEditedEvent(ObjectModifiedEvent):
    """An object is being edited, i.e. modified after the first save
    """
    implements(IObjectEditedEvent)

class WebDAVObjectEditedEvent(ObjectEditedEvent):
    """An object is being edited via WebDAV
    """
    implements(IWebDAVObjectEditedEvent)

class EditBegunEvent(ObjectEvent):
    """An edit operation was begun
    """
    implements(IEditBegunEvent)

class EditCancelledEvent(ObjectEvent):
    """An edit operation was cancelled
    """
    implements(IEditCancelledEvent)



## Script (Python) "content_edit_impl"
##title=Edit content
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=state, id=''
##

from Products.Archetypes import PloneMessageFactory as _
from Products.Archetypes.utils import addStatusMessage

REQUEST = context.REQUEST
old_id = context.getId()

try:
    new_context = context.portal_factory.doCreate(context, id)
except AttributeError:
    # Fallback for AT + plain CMF where we don't have a portal_factory
    new_context = context
new_context.processForm()

# Get the current language and put it in request/LANGUAGE
form = REQUEST.form
if 'current_lang' in form:
    form['language'] = form.get('current_lang')

portal_status_message = _(u'Changes saved.')

# handle navigation for multi-page edit forms
next = not REQUEST.get('form.button.next', None) is None
previous = not REQUEST.get('form.button.previous', None) is None
fieldset = REQUEST.get('fieldset', None)
schemata = new_context.Schemata()

if next or previous:
    s_names = [s for s in schemata.keys() if s != 'metadata']

    if previous:
        s_names.reverse()

    next_schemata = None
    try:
        index = s_names.index(fieldset)
    except ValueError:
        raise 'Non-existing fieldset: %s' % fieldset
    else:
        index += 1
        if index < len(s_names):
            next_schemata = s_names[index]
            addStatusMessage(REQUEST, portal_status_message)
            return state.set(status='next_schemata',
                             context=new_context,
                             fieldset=next_schemata)

    if next_schemata != None:
        addStatusMessage(REQUEST, portal_status_message)
        return state.set(status='next_schemata',
                 context=new_context,
                 fieldset=next_schemata)
    else:
        raise 'Unable to find next field set after %s' % fieldset

env = state.kwargs
reference_source_url = env.get('reference_source_url')
if reference_source_url is not None:
    reference_source_url = env['reference_source_url'].pop()
    reference_source_field = env['reference_source_field'].pop()
    reference_source_fieldset = env['reference_source_fieldset'].pop()
    portal = context.portal_url.getPortalObject()
    reference_obj = portal.restrictedTraverse(reference_source_url)
    portal_status_message = _(u'message_reference_added',
                              default=u'Reference added.')

    edited_reference_message = _(u'message_reference_edited',
                                 default=u'Reference edited.')

    kwargs = {
        'status': 'success_add_reference',
        'context': reference_obj,
        'fieldset': reference_source_fieldset,
        'field': reference_source_field,
        'reference_focus': reference_source_field,
        }
    addStatusMessage(REQUEST, portal_status_message)
    return state.set(**kwargs)

if state.errors:
    errors = state.errors
    s_items = [(s, schemata[s].keys()) for s in schemata.keys()]
    fields = []
    for s, f_names in s_items:
        for f_name in f_names:
            fields.append((s, f_name))
    for s_name, f_name in fields:
        if f_name in errors:
            REQUEST.set('fieldset', s_name)
            addStatusMessage(REQUEST, portal_status_message)
            return state.set(
                status='failure',
                context=new_context)

if not state.errors:
    from Products.Archetypes.utils import transaction_note
    transaction_note('Edited %s %s at %s' % (new_context.meta_type,
                                             new_context.title_or_id(),
                                             new_context.absolute_url()))

addStatusMessage(REQUEST, portal_status_message)
return state.set(status='success',
                 context=new_context)



## Script (Python) "unicodeEncode"
##title=Return an encoded string using the site charset
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=value, site_charset=None

# Recursively deal with sequences
tuplevalue = same_type(value, ())
if (tuplevalue or same_type(value, [])):
    encoded = [context.unicodeEncode(v) for v in value]
    if tuplevalue:
        encoded = tuple(encoded)
    return encoded

if not isinstance(value, basestring):
    value = str(value)

if site_charset is None:
    site_charset = context.getCharset()

if same_type(value, ''):
    value = unicode(value, site_charset)

# don't try to catch unicode error here
# if one occurs, that means the site charset must be changed !
return value.encode(site_charset)



################################################################################
#
# Copyright (c) 2002-2005, Benjamin Saller <bcsaller@ideasuite.com>, and
#                              the respective authors. All rights reserved.
# For a list of Archetypes contributors see docs/CREDITS.txt.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the author nor the names of its contributors may be used
#   to endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
################################################################################

from UserDict import DictMixin

from BTrees.OOBTree import OOBTree
from Acquisition import Explicit
from Acquisition import aq_base
from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass

from Products.Archetypes.interfaces.annotations import IATAnnotations
from Products.Archetypes.interfaces.annotations import IATAnnotatable

from zope.interface import implements
from zope.component import adapts

# annotation keys
AT_ANN_STORAGE = 'Archetypes.storage.AnnotationStorage'
AT_MD_STORAGE = 'Archetypes.storage.MetadataAnnotationStorage'
AT_FIELD_MD = 'Archetypes.field.Metadata'
AT_REF = 'Archetypes.referenceEngine.Reference'

# all keys so someone can test against this list
AT_ANN_KEYS = (AT_ANN_STORAGE, AT_MD_STORAGE, AT_FIELD_MD, AT_REF)


class ATAnnotations(DictMixin, Explicit):
    """Store annotations in the '__annotations__' attribute on a IATAnnotatable
       object.
    """
    implements(IATAnnotations)
    adapts(IATAnnotatable)

    security = ClassSecurityInfo()
    security.declareObjectPrivate()

    def __init__(self, obj):
        self._obj = aq_base(obj)

    # basic methods required for DictMixin

    def __nonzero__(self):
        return bool(getattr(self._obj, '__annotations__', False))

    def get(self, key, default=None):
        annotations = getattr(self._obj, '__annotations__', None)
        if not annotations:
            return default

        return annotations.get(key, default)

    def __getitem__(self, key):
        annotations = getattr(self._obj, '__annotations__', None)
        if annotations is None:
            raise KeyError, key

        return annotations[key]

    def keys(self):
        annotations = getattr(self._obj, '__annotations__', None)
        if annotations is None:
            return []

        return annotations.keys()

    def __setitem__(self, key, value):
        if not isinstance(key, basestring):
            raise TypeError('ATAnnotations key must be a string')
        try:
            annotations = self._obj.__annotations__
        except AttributeError:
            annotations = self._obj.__annotations__ = OOBTree()

        annotations[key] = value

    def __delitem__(self, key):
        try:
            annotation = self._obj.__annotations__
        except AttributeError:
            raise KeyError, key

        del annotation[key]

    # additional methods

    def set(self, key, value):
        self[key] = value

    def getSubkey(self, key, subkey, default=None):
        # Get annotations using a key and one subkey.
        if isinstance(subkey, basestring):
            k = '%s-%s' % (key, subkey)
            return self.get(k, default)
        else:
            raise TypeError('Invalid subkey type %s, must be string type' % type(subkey))

    def setSubkey(self, key, value, subkey):
        # Stores data using a key and one subkey.
        if isinstance(subkey, basestring):
            k = '%s-%s' % (key, subkey)
            self[k] = value
        else:
            raise TypeError('Invalid subkey type %s, must be string type' % type(subkey))

    def delSubkey(self, key, subkey):
        # Removes a subkey.
        if isinstance(subkey, basestring):
            k = '%s-%s' % (key, subkey)
            del self[k]
        else:
            raise TypeError('Invalid subkey type %s, must be string type' % type(subkey))

    def hasSubkey(self, key, subkey):
        # Checks for the existence of a sub key.
        if isinstance(subkey, basestring):
            k = '%s-%s' % (key, subkey)
            return k in self
        else:
            raise TypeError('Invalid subkey type %s, must be string type' % type(subkey))

    def getObject(self):
        return self._obj

    def getAnnotationObject(self):
        try:
            return self._obj.__annotations__
        except AttributeError:
            return None

    # DictMixin does define the following methods:
    #def __iter__(self):
    #def has_key(self, key):
    #def __contains__(self, key):
    #def iteritems(self):
    #def iterkeys(self):
    #def itervalues(self):
    #def values(self):
    #def items(self):
    #def clear(self):
    #def setdefault(self, key, default):
    #def pop(self, key, *args):
    #def popitem(self):
    #def update(self, other):
    #def get(self, key, default=None):
    #def __repr__(self):
    #def __cmp__(self, other):
    #def __len__(self):

InitializeClass(ATAnnotations)


def getAnnotation(obj):
    """Get an ATAnnotation object for obj
    """
    return ATAnnotations(obj).__of__(obj)



################################################################################
#
# Copyright (c) 2002-2005, Benjamin Saller <bcsaller@ideasuite.com>, and
#                              the respective authors. All rights reserved.
# For a list of Archetypes contributors see docs/CREDITS.txt.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the author nor the names of its contributors may be used
#   to endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
################################################################################
"""
Unittests for a copying/cutting and pasting archetypes objects.
"""

from unittest import TestSuite, makeSuite

import os

import transaction
from Acquisition import aq_base

from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import noSecurityManager

from Products.Archetypes.tests.atsitetestcase import ATSiteTestCase
from Products.Archetypes.tests.utils import makeContent
from Products.Archetypes.tests.atsitetestcase import portal_owner
from Products.Archetypes.tests.atsitetestcase import portal_name
from Products.Archetypes.tests.atsitetestcase import default_user

from Products.Archetypes.tests.utils import PACKAGE_HOME


class CutPasteCopyPasteTests(ATSiteTestCase):

    def test_copy_and_paste(self):
        ffrom = makeContent(self.folder, portal_type='SimpleFolder', id='cangucu')
        tourist = makeContent(ffrom, portal_type='Fact', id='tourist')
        fto = makeContent(self.folder, portal_type='SimpleFolder', id='london')
        self.assertFalse('tourist' not in ffrom.contentIds())

        #make sure we have _p_jar
        transaction.savepoint(optimistic=True)
        cb = ffrom.manage_copyObjects(ffrom.contentIds())
        fto.manage_pasteObjects(cb)
        self.assertFalse('tourist' not in ffrom.contentIds())
        self.assertFalse('tourist' not in fto.contentIds())

    def test_cut_and_paste(self):
        ffrom = makeContent(self.folder, portal_type='SimpleFolder', id='cangucu')
        tourist = makeContent(ffrom, portal_type='Fact', id='tourist')
        fto = makeContent(self.folder, portal_type='SimpleFolder', id='london')
        self.assertFalse('tourist' not in ffrom.contentIds())

        #make sure we have _p_jar
        transaction.savepoint(optimistic=True)
        cb = ffrom.manage_cutObjects(ffrom.contentIds())
        fto.manage_pasteObjects(cb)
        self.assertFalse('tourist' in ffrom.contentIds())
        self.assertFalse('tourist' not in fto.contentIds())


class PortalCopyTests(ATSiteTestCase):

    def afterSetUp(self):
        ATSiteTestCase.afterSetUp(self)
        self.setRoles(['Manager'])

        imgpath = os.path.join(PACKAGE_HOME, os.pardir, 'tool.gif')
        self._image = open(imgpath).read()

        portal = self.portal

        portal.invokeFactory('DDocument', id='document')
        doc = portal.document
        doc.setBody('testdata', mimetype='text/x-rst')
        doc.setImage(self._image, mimetype='image/gif')

    def _test_doc(self, doc):
        bodyfield = doc.getField('body')
        imagefield = doc.getField('image')

        self.assertEqual(doc.getContentType(), 'text/x-rst')

        self.assertEqual(doc.getRawBody(), 'testdata')
        self.assertTrue(doc.getImage().data, self._image)

        self.assertTrue(bodyfield.getContentType(doc), 'text/x-rst')

    def test_created_doc(self):
        portal = self.portal
        self.assertTrue(portal, 'document')
        doc = portal.document
        self._test_doc(doc)

    def test_clone_portal(self):
        app = self.app
        user = app.acl_users.getUserById(portal_owner).__of__(app.acl_users)
        newSecurityManager(None, user)
        app.manage_clone(self.portal, 'newportal')
        noSecurityManager()
        transaction.savepoint(optimistic=True)

        self.assertTrue(hasattr(aq_base(app), 'newportal'))
        self.newportal = app.newportal
        # check if we really have new portal!
        self.assertFalse(aq_base(self.newportal) is aq_base(self.portal))
        self.assertNotEqual(aq_base(self.newportal), aq_base(self.portal))

        self.assertTrue(hasattr(aq_base(self.newportal), 'document'))
        doc = self.newportal.document
        self._test_doc(doc)

    def test_copy_paste_portal(self):
        app = self.app
        user = app.acl_users.getUserById('portal_owner').__of__(app.acl_users)
        newSecurityManager(None, user)
        cp = app.manage_copyObjects(ids=[portal_name])
        app.manage_pasteObjects(cb_copy_data=cp)

        noSecurityManager()
        transaction.savepoint(optimistic=True)

        self.assertTrue(hasattr(aq_base(self.app), 'copy_of_%s' % portal_name))
        self.newportal = getattr(self.app, 'copy_of_%s' % portal_name)
        # check if we really have new portal!
        self.assertFalse(aq_base(self.newportal) is aq_base(self.portal))
        self.assertNotEqual(aq_base(self.newportal), aq_base(self.portal))

        self.assertTrue(hasattr(aq_base(self.newportal), 'document'))
        doc = self.newportal.document
        self._test_doc(doc)

    def test_cut_paste_portal(self):
        app = self.app
        user = app.acl_users.getUserById(portal_owner).__of__(app.acl_users)
        newSecurityManager(None, user)
        cp = app.manage_cutObjects(ids=[portal_name])
        app.manage_pasteObjects(cb_copy_data=cp)

        noSecurityManager()
        transaction.savepoint(optimistic=True)

        self.assertTrue(hasattr(aq_base(self.app), portal_name))
        self.newportal = getattr(self.app, portal_name)

        self.assertTrue(hasattr(aq_base(self.newportal), 'document'))
        doc = self.newportal.document
        self._test_doc(doc)

    def test_copy_paste_sets_ownership(self):
        # Copy/pasting a File should set new ownership including local roles
        # borrowed from CMFCore tests

        # First, create a new manager user
        uf = self.portal.acl_users
        uf._doAddUser('manager1', 'secret', ['Manager'], [])
        member = uf.getUser(default_user).__of__(uf)
        manager1 = uf.getUser('manager1').__of__(uf)
        member_area = self.portal.Members[default_user]

        # Switch to the manager user context and plant a content item into
        # the member user's member area
        self.login('manager1')
        member_area.invokeFactory('DDocument', id='test_file')

        # Switch to "member" context now and try to copy and paste the
        # content item created by "manager1"
        self.login(default_user)
        cb = member_area.manage_copyObjects(['test_file'])
        member_area.manage_pasteObjects(cb)

        # Now test executable ownership and "owner" local role
        # "member" should have both.
        file_ob = member_area.copy_of_test_file
        self.assertEqual(aq_base(file_ob.getOwner().getId()), aq_base(member).getId())
        self.assertTrue('Owner' in
                            file_ob.get_local_roles_for_userid(default_user))

    def test_copy_paste_resets_workflow(self):
        # Copy/pasting a File should reset workflow to the default state

        # This test depends on the default wf, assume CMFDefault if plone
        # isn't present
        wf_tool = self.portal.portal_workflow
        if 'plone_workflow' in wf_tool.objectIds():
            wf_id = 'plone_workflow'
            def_state = 'visible'
        else:
            wf_id = 'default_workflow'
            def_state = 'private'

        wf_tool.setChainForPortalTypes(('DDocument',), (wf_id,))
        self.folder.invokeFactory('DDocument', id='test_file')

        file = self.folder.test_file

        self.assertEqual(wf_tool.getInfoFor(file, 'review_state'), def_state)
        wf_tool.doActionFor(file, 'publish')
        self.assertEqual(wf_tool.getInfoFor(file, 'review_state'),
                                                                 'published')

        cb = self.folder.manage_copyObjects(['test_file'])
        self.folder.manage_pasteObjects(cb)

        file_copy = self.folder.copy_of_test_file
        self.assertEqual(wf_tool.getInfoFor(file_copy, 'review_state'),
                                                                   def_state)

    def test_cut_paste_preserves_workflow(self):
        # Cut/pasting a File should preserve workflow state

        # This test depends on the default wf, assume CMFDefault if plone
        # isn't present
        wf_tool = self.portal.portal_workflow
        if 'plone_workflow' in wf_tool.objectIds():
            wf_id = 'plone_workflow'
            def_state = 'visible'
        else:
            wf_id = 'default_workflow'
            def_state = 'private'

        wf_tool.setChainForPortalTypes(('DDocument',), (wf_id,))
        self.folder.invokeFactory('DDocument', id='test_file')
        self.folder.invokeFactory('Folder', id='sub')

        file = self.folder.test_file

        self.assertEqual(wf_tool.getInfoFor(file, 'review_state'), def_state)
        wf_tool.doActionFor(file, 'publish')
        self.assertEqual(wf_tool.getInfoFor(file, 'review_state'),
                                                                 'published')

        transaction.savepoint(optimistic=True)
        cb = self.folder.manage_cutObjects(['test_file'])
        self.folder.sub.manage_pasteObjects(cb)

        file_copy = self.folder.sub.test_file
        self.assertEqual(wf_tool.getInfoFor(file_copy, 'review_state'),
                                                                 'published')

    def test_copy_handles_talkback_Items(self):
        # We need to ensure that the manage_after* methods are called on
        # subobjects of non-folderish objects
        doc = self.portal.document
        dtool = self.portal.portal_discussion
        cat = self.portal.portal_catalog

        doc.allowDiscussion('1')
        tb = dtool.getDiscussionFor(doc)
        tb.createReply(title='Stupendous', text='silly', Creator='admin')

        # Creating the reply should have cataloged it in manage_afterAdd
        results = cat(Title='Stupendous')
        self.assertEqual(len(results), 1)

        cp = self.portal.manage_copyObjects(ids=['document'])
        self.folder.manage_pasteObjects(cp)
        doc_copy = self.folder.document

        tb = dtool.getDiscussionFor(doc_copy)
        self.assertTrue(tb.hasReplies(doc_copy), "Discussion not copied")

        # Copying doc should have cataloged the reply in manage_afterClone
        results = cat(Title='Stupendous')
        self.assertEqual(len(results), 2)

    # not sure where else to put this
    def test_delete_handles_talkback_Items(self):
        doc = self.portal.document
        dtool = self.portal.portal_discussion
        cat = self.portal.portal_catalog

        doc.allowDiscussion('1')
        tb = dtool.getDiscussionFor(doc)
        tb.createReply(title='Stupendous', text='silly', Creator='admin')

        # Creating the reply should have cataloged it in manage_afterAdd
        results = cat(Title='Stupendous')
        self.assertEqual(len(results), 1)

        self.portal.manage_delObjects(ids=['document'])

        # Deleting the doc should have removed the discussions from the
        # catalog
        results = cat(Title='Stupendous')
        self.assertEqual(len(results), 0)


def test_suite():
    suite = TestSuite()
    suite.addTest(makeSuite(CutPasteCopyPasteTests))
    suite.addTest(makeSuite(PortalCopyTests))
    return suite



from Testing.ZopeTestCase.layer import ZopeLite

from zope.testing.cleanup import cleanUp

from Products.CMFCore.testing import FunctionalZCMLLayer
from Products.CMFTestCase import layer as cmf_layer

# BBB Zope 2.12
try:
    from Zope2.App import zcml
except ImportError:
    from Products.Five import zcml


class ZCML(FunctionalZCMLLayer):

    def setUp(cls):
        '''Sets up the CA.'''
        import Products.CMFDefault
        zcml.load_config('configure.zcml', Products.CMFDefault)

        import Products.CMFCalendar
        zcml.load_config('configure.zcml', Products.CMFCalendar)

        import Products.DCWorkflow
        zcml.load_config('configure.zcml', Products.DCWorkflow)

        import Products.Archetypes
        zcml.load_config('configure.zcml', Products.Archetypes)

        import Products.PlacelessTranslationService
        zcml.load_config('configure.zcml', Products.PlacelessTranslationService)

        import plone.uuid
        zcml.load_config('configure.zcml', plone.uuid)

    setUp = classmethod(setUp)

    def tearDown(cls):
        '''Cleans up the CA.'''
        cleanUp()
    tearDown = classmethod(tearDown)


class ArchetypesSite(ZCML):

    def setUp(cls):
        '''Sets up the CA.'''
        for func, args, kw in cmf_layer._deferred_setup:
            func(*args, **kw)

    setUp = classmethod(setUp)

    def tearDown(cls):
        '''Cleans up the CA.'''
        for func, args, kw in cmf_layer._deferred_cleanup:
            func(*args, **kw)

        # cleanUp()
    tearDown = classmethod(tearDown)



from unittest import TestSuite, makeSuite

from zope.component import getMultiAdapter

from Products.Archetypes.tests.atsitetestcase import ATSiteTestCase
from Products.Archetypes.atapi import DisplayList
from Products.Archetypes.atapi import IntDisplayList


class UtilsMethodsTests(ATSiteTestCase):
    """Test the different methods in browser.utils view."""

    def test_translate_vocab_with_special_chars(self):
        vocab = DisplayList((('Sp\xc3\xa9cial char key', 'Sp\xc3\xa9cial char value'),
                             ('normal_key', 'With sp\xc3\xa9cial char'), ))
        utilsView = getMultiAdapter((self.portal, self.portal.REQUEST), name='at_utils')
        # Note that due to the test setup, the result is expected to
        # be u'[[domain][translation]]'.
        self.assertEqual(utilsView.translate(vocab, value='Sp\xc3\xa9cial char key'),
                         u'[[plone][Sp\xe9cial char value]]')
        self.assertEqual(utilsView.translate(vocab, value='normal_key'),
                         u'[[plone][With sp\xe9cial char]]')
        # Try with unicode as input value
        self.assertEqual(utilsView.translate(vocab, value=u'Sp\xe9cial char key'),
                         u'[[plone][Sp\xe9cial char value]]')
        self.assertEqual(utilsView.translate(vocab, value=u'normal_key'),
                         u'[[plone][With sp\xe9cial char]]')

    def test_translate_integer_display_list(self):
        vocab = IntDisplayList(((1, 'Sp\xc3\xa9cial char value'),
                                (2, 'two'),
                                (3, 42), ))
        utilsView = getMultiAdapter((self.portal, self.portal.REQUEST), name='at_utils')
        self.assertEqual(utilsView.translate(vocab, value=1),
                         u'[[plone][Sp\xe9cial char value]]')
        self.assertEqual(utilsView.translate(vocab, value=2),
                         u'[[plone][two]]')
        self.assertEqual(utilsView.translate(vocab, value=3),
                         u'[[plone][42]]')

    def test_translate_empty(self):
        vocab = DisplayList((('one', 'One'),
                             ('two', 'Two'), ))
        utilsView = getMultiAdapter((self.portal, self.portal.REQUEST), name='at_utils')
        self.assertEqual(utilsView.translate(vocab, value=''), u'')
        self.assertEqual(utilsView.translate(vocab, value=None), u'')
        self.assertEqual(utilsView.translate(vocab, value=[]), u'')
        self.assertEqual(utilsView.translate(vocab, value=()), u'')
        self.assertEqual(utilsView.translate(vocab, value=set()), u'')


def test_suite():
    suite = TestSuite()
    suite.addTest(makeSuite(UtilsMethodsTests))
    return suite



"""Field properties based on Archetypes schema
"""

from DateTime import DateTime
from datetime import datetime
from zope.datetime import parseDatetimetz

from zope.site.hooks import getSite

class ATFieldProperty(object):
    """Field properties based on Archetypes schema

    These properties can only be used on Archetypes objects. They delegate
    to schema.getField(fieldname).get() and set().

    You can use it in your type as follows. The name of the field does
    not need to coincide with the field-property name, but this is probably
    sensible. However, AttributeStorage will interfere here, so we explicitly
    use annoation storage.

        >>> import string
        >>> from Products.Archetypes.atapi import *
        >>> class MyContent(BaseObject):
        ...     portal_type = meta_type = 'MyContent'
        ...     schema = Schema((
        ...         StringField('some_field', storage=AnnotationStorage()),
        ...         StringField('_other_field'),
        ...         ))
        ...
        ...     some_field = ATFieldProperty('some_field')
        ...     other_field = ATFieldProperty('_other_field')
        ...     upper_lower = ATFieldProperty('_other_field',
        ...         get_transform=string.upper, set_transform=string.lower)

        >>> registerType(MyContent, 'Archetypes')

    Now, get and set operations on the fieldproperty behave the same way as
    the mutator and accessor.

        >>> foo = MyContent('foo')
        >>> foo.some_field
        ''
        >>> foo.some_field = "Bar"
        >>> foo.some_field
        'Bar'
        >>> foo.getField('some_field').get(foo)
        'Bar'

    The old-style mutator and accessors still work, of course

        >>> foo.getSome_field()
        'Bar'

        >>> foo.setSome_field("Baz")
        >>> foo.some_field
        'Baz'

    Here is an example using the default AttributeStorage. In this case, we
    need different names for the AT field name and the properity, because
    AttributeStorage will use the field name as the attribute name. If
    you don't do this, you may get infinite recursion!

        >>> foo.other_field = "Hello"
        >>> foo.other_field
        'Hello'
        >>> foo.get_other_field()
        'Hello'
        >>> foo.set_other_field("Good bye")
        >>> foo.other_field
        'Good bye'

    Finally, the get_transform and set_transform arguments can be used to
    perform transformations on the retrieved value and the value before it
    is set, respectively. The field upper_lower uses string.upper() on the
    way out and string.lower() on the way in.

        >>> foo.upper_lower = "MiXeD"
        >>> foo.upper_lower
        'MIXED'
        >>> foo.get_other_field()
        'mixed'
        >>> foo.set_other_field('UpPeRaNdLoWeR')
        >>> foo.upper_lower
        'UPPERANDLOWER'

    A less frivolous example of this functionality can be seen in the
    ATDateTimeFieldProperty class below.
    """

    def __init__(self, name, get_transform=None, set_transform=None):
        self._name = name
        self._get_transform = get_transform
        self._set_transform = set_transform

    def __get__(self, inst, klass):
        if inst is None:
            return self
        field = inst.getField(self._name)
        if field is None:
            raise KeyError("Cannot find field with name %s" % self._name)
        value = field.get(inst)
        if self._get_transform is not None:
            value = self._get_transform(value)
        return value

    def __set__(self, inst, value):
        field = inst.getField(self._name)
        if field is None:
            raise KeyError("Cannot find field with name %s" % self._name)
        if self._set_transform is not None:
            value = self._set_transform(value)
        field.set(inst, value)

class ATToolDependentFieldProperty(ATFieldProperty):
    """A version of the field property type which is able to acquire
    tools. This uses a not-very-nice acquisition hack, and is not
    generalisable to all acquisition-dependent operations, but should work
    for tools in the portal root.

        >>> from Products.Archetypes.atapi import *
        >>> class MyContent(BaseContent):
        ...     portal_type = meta_type = 'MyContent'
        ...     schema = Schema((
        ...         ReferenceField('some_field', multiValued=True,
        ...                        relationship='foo', storage=AnnotationStorage()),
        ...         ))
        ...
        ...     some_field = ATToolDependentFieldProperty('some_field')

        >>> registerType(MyContent, 'Archetypes')

        >>> self.portal._setOb('foo', MyContent('foo'))
        >>> foo = getattr(self.portal, 'foo')

        >>> self.portal._setOb('bar', MyContent('bar'))
        >>> bar = getattr(self.portal, 'bar')
        >>> bar._at_uid = 123456

    These lines would fail with AttributeError: reference_catalog if it used
    the standard accessor.

        >>> foo.some_field
        []
        >>> foo.some_field = [bar]
        Traceback (most recent call last):
        ...
        ReferenceException: 123456 not referenceable
        >>> foo.some_field
        []
    """

    def __init__(self, name, get_transform=None, set_transform=None):
        self._name = name
        self._get_transform = get_transform
        self._set_transform = set_transform

    def __get__(self, inst, klass):
        if inst is None:
            return self
        field = inst.getField(self._name)
        if field is None:
            raise KeyError("Cannot find field with name %s" % self._name)
        value = field.get(inst.__of__(getSite()))
        if self._get_transform is not None:
            value = self._get_transform(value)
        return value

    def __set__(self, inst, value):
        field = inst.getField(self._name)
        if field is None:
            raise KeyError("Cannot find field with name %s" % self._name)
        if self._set_transform is not None:
            value = self._set_transform(value)
        field.set(inst.__of__(getSite()), value)

class ATReferenceFieldProperty(ATToolDependentFieldProperty):
    """A more friendly/use-case-specific name for ATReferenceFieldProperty.
    """

class ATDateTimeFieldProperty(ATFieldProperty):
    """A field property for DateTime fields. This takes care of converting
    to and from a Python datetime.

        >>> from Products.Archetypes.atapi import *
        >>> class MyContent(BaseObject):
        ...     portal_type = meta_type = 'MyContent'
        ...     schema = Schema((
        ...         DateTimeField('date_field', storage=AnnotationStorage()),
        ...         ))
        ...
        ...     date_field = ATDateTimeFieldProperty('date_field')

        >>> registerType(MyContent, 'Archetypes')

    We can now get and set date/times.

        >>> from datetime import datetime
        >>> target_date = datetime(2007, 4, 9, 12, 3, 12)

        >>> foo = MyContent('foo')
        >>> foo.date_field = target_date
        >>> foo.date_field
        datetime.datetime(2007, 4, 9, 12, 3, 12, ...)

        >>> foo.getDate_field().ISO8601()
        '2007-04-09T12:03:12'

        >>> foo.setDate_field(DateTime('2007-04-10 13:11:01'))
        >>> foo.date_field
        datetime.datetime(2007, 4, 10, 13, 11, 1, ...)
    """

    def __init__(self, name):
        super(ATDateTimeFieldProperty, self).__init__(name, self._zope2python_dt, self._python2zope_dt)

    def _zope2python_dt(self, zope_dt):
        if zope_dt is None:
            return None
        return parseDatetimetz(zope_dt.ISO8601())

    def _python2zope_dt(self, python_dt):
        if python_dt is None:
            return None
        return DateTime(python_dt.isoformat())



################################################################################
#
# Copyright (c) 2002-2005, Benjamin Saller <bcsaller@ideasuite.com>, and
#                              the respective authors. All rights reserved.
# For a list of Archetypes contributors see docs/CREDITS.txt.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the author nor the names of its contributors may be used
#   to endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
################################################################################
"""
"""

from Products.Archetypes.tests.attestcase import ATTestCase

from Products.Archetypes.atapi import StringField
from Products.Archetypes.Schema import ManagedSchema


class SchemataManipulationTest(ATTestCase):

    def afterSetUp(self):
        self.schema = ManagedSchema(
               (StringField('a', schemata='waldi'),
                StringField('d', schemata='nasbaer'),
                StringField('x', schemata='edgar'),
                StringField('b', schemata='waldi'),
                StringField('e', schemata='nasbaer'),
                StringField('y', schemata='edgar'),
                StringField('c', schemata='waldi'),
                StringField('f', schemata='nasbaer'),
                StringField('z', schemata='edgar')
            ))

    def fields2names(self, fields):
        return [f.getName() for f in fields]

    def testBasic(self):
        self.assertEqual(self.fields2names(self.schema.fields()),
                        ['a', 'd', 'x', 'b', 'e', 'y', 'c', 'f', 'z'])
        self.assertEqual(self.schema.getSchemataNames(), ['waldi', 'nasbaer', 'edgar'])
        self.schema.addField(StringField('p', schemata='waldi'))
        self.schema.addField(StringField('hello_world', schemata='helloworld'))
        self.schema.addField(StringField('hello_world1', schemata='helloworld1'))
        self.assertEqual(self.schema.getSchemataNames(), ['waldi', 'nasbaer', 'edgar', 'helloworld', 'helloworld1'])

    def testSchemataFields(self):
        self.assertEqual(self.fields2names(self.schema.getSchemataFields('waldi')),
                        ['a', 'b', 'c'])
        self.assertEqual(self.fields2names(self.schema.getSchemataFields('nasbaer')),
                        ['d', 'e', 'f'])
        self.assertEqual(self.fields2names(self.schema.getSchemataFields('edgar')),
                        ['x', 'y', 'z'])

    def testDelField(self):
        self.schema.delField('x')
        self.schema.delField('b')
        self.schema.delField('z')
        self.assertEqual(self.fields2names(self.schema.fields()),
                        ['a', 'd', 'e', 'y', 'c', 'f'])
        self.schema.addField(StringField('z'))
        self.schema.addField(StringField('b'))
        self.schema.addField(StringField('x'))
        self.assertEqual(self.fields2names(self.schema.fields()),
                        ['a', 'd', 'e', 'y', 'c', 'f', 'z', 'b', 'x'])
        self.schema.delField('b')
        self.schema.delField('z')
        self.schema.delField('x')
        self.assertEqual(self.fields2names(self.schema.fields()),
                        ['a', 'd', 'e', 'y', 'c', 'f'])

    def testDelSchemata(self):
        self.schema.delSchemata('nasbaer')
        self.assertEqual(self.schema.getSchemataNames(), ['waldi', 'edgar'])
        self.schema.addField(StringField('hello_world', schemata='helloworld'))
        self.assertEqual(self.schema.getSchemataNames(), ['waldi', 'edgar', 'helloworld'])

    def testAddSchemata(self):
        self.schema.addSchemata('otto')
        self.assertEqual(self.schema.getSchemataNames(), ['waldi', 'nasbaer', 'edgar', 'otto'])
        self.assertEqual(len(self.schema.getSchemataFields('otto')), 1)

    def testFieldChangeSchemata(self):
        self.schema.changeSchemataForField('z', 'otto')
        self.assertEqual(self.schema.getSchemataNames(), ['waldi', 'nasbaer', 'edgar', 'otto'])
        self.assertEqual(self.fields2names(self.schema.getSchemataFields('otto')), ['z'])
        self.schema.changeSchemataForField('z', 'waldi')
        self.assertEqual(self.schema.getSchemataNames(), ['waldi', 'nasbaer', 'edgar'])

    def testMoveSchemata1(self):
        self.schema.moveSchemata('waldi', -1)
        self.assertEqual(self.schema.getSchemataNames(), ['waldi', 'nasbaer', 'edgar'])
        self.schema.moveSchemata('waldi', 1)
        self.assertEqual(self.schema.getSchemataNames(), ['nasbaer', 'waldi', 'edgar'])
        self.schema.moveSchemata('waldi', 1)
        self.assertEqual(self.schema.getSchemataNames(), ['nasbaer', 'edgar', 'waldi'])
        self.schema.moveSchemata('waldi', 1)
        self.assertEqual(self.schema.getSchemataNames(), ['nasbaer', 'edgar', 'waldi'])

    def testMoveSchemata2(self):
        self.schema.moveSchemata('edgar', 1)
        self.assertEqual(self.schema.getSchemataNames(), ['waldi', 'nasbaer', 'edgar'])
        self.schema.moveSchemata('edgar', -1)
        self.assertEqual(self.schema.getSchemataNames(), ['waldi', 'edgar', 'nasbaer'])
        self.schema.moveSchemata('edgar', -1)
        self.assertEqual(self.schema.getSchemataNames(), ['edgar', 'waldi', 'nasbaer'])
        self.schema.moveSchemata('edgar', -1)
        self.assertEqual(self.schema.getSchemataNames(), ['edgar', 'waldi', 'nasbaer'])

    def testMoveField(self):
        self.assertEqual(self.fields2names(self.schema.getSchemataFields('waldi')), ['a', 'b', 'c'])
        self.schema.moveField('a', -1)
        self.assertEqual(self.fields2names(self.schema.getSchemataFields('waldi')), ['a', 'b', 'c'])
        self.schema.moveField('a', 1)
        self.assertEqual(self.fields2names(self.schema.getSchemataFields('waldi')), ['b', 'a', 'c'])
        self.schema.moveField('a', 1)
        self.assertEqual(self.fields2names(self.schema.getSchemataFields('waldi')), ['b', 'c', 'a'])

    def testReplaceField(self):
        f1 = StringField('f1')
        f2 = StringField('f2')
        f3 = StringField('f3')
        self.schema.replaceField('a', f1)
        self.schema.replaceField('e', f2)
        self.schema.replaceField('z', f3)
        self.assertEqual(self.fields2names(self.schema.fields()),
                        ['f1', 'd', 'x', 'b', 'f2', 'y', 'c', 'f', 'f3'])


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(SchemataManipulationTest))
    return suite



from Acquisition import aq_inner
from Products.Five import BrowserView


# map from mimetypes used in allowable_content_types to mimetypes that
# are stored in the base unit
MIMETYPES_MAPPING = {
    'text/x-python': 'text/python-source',
    'text/restructured': 'text/x-rst',
}


class SelectionWidget(BrowserView):
    """View used in Archetypes language and selection widget.

    We start with a browser view for this widget.  We use a test
    request and some very simple content to initialize it.

    >>> from zope.publisher.browser import TestRequest
    >>> class SimpleContent(object):
    ...     def getCharset(self):
    ...         return 'utf-8'
    >>> widget = SelectionWidget(SimpleContent(), TestRequest())

    Test with a simple vocabulary

    >>> vocab = ('a', 'b', 'c')
    >>> widget.getSelected(vocab, 'a')
    ['a']
    >>> widget.getSelected(vocab, 'A')
    []
    >>> widget.getSelected(vocab, 'd')
    []

    >>> vocab = ('b', 'a', 'd', 'c')
    >>> widget.getSelected(vocab, ('b', 'c'))
    ['b', 'c']

    Test with a DisplayList

    >>> from Products.Archetypes.utils import DisplayList
    >>> friends = DisplayList([('Monty Python', u'monty'),
    ...                        (u'Guido van Rossum', u'guido')])
    >>> widget.getSelected(friends, 'monty')
    []
    >>> widget.getSelected(friends, u'guido')
    []
    >>> widget.getSelected(friends, 'Spanish Inquisition')
    []

    getSelected is used to get a list of selected vocabulary items.
    In the widget, we repeat on the vocabulary, comparing
    its values with those returned by getSelected. So,
    we always return the same encoding as in the vocabulary.

    >>> widget.getSelected(friends, u'Monty Python')
    ['Monty Python']
    >>> widget.getSelected(friends, 'Monty Python')
    ['Monty Python']
    >>> widget.getSelected(friends, u'Guido van Rossum')
    [u'Guido van Rossum']
    >>> widget.getSelected(friends, 'Guido van Rossum')
    [u'Guido van Rossum']

    Test with an IntDisplayList:

    >>> from Products.Archetypes.utils import IntDisplayList
    >>> quarter_vocabulary = IntDisplayList([(0, '0'), (15, '15'),
    ...                                      (30, '30'), (45, '45')])
    >>> widget.getSelected(quarter_vocabulary, 5)
    []
    >>> widget.getSelected(quarter_vocabulary, 0)
    [0]
    >>> widget.getSelected(quarter_vocabulary, 15)
    [15]
    >>> widget.getSelected(quarter_vocabulary, '15')
    [15]
    >>> widget.getSelected(quarter_vocabulary, 'wrongdata')
    []
    >>> widget.getSelected(quarter_vocabulary, None)
    []

    """

    def getSelected(self, vocab, value):

        context = aq_inner(self.context)

        site_charset = context.getCharset()

        # compile a dictionary from the vocabulary of
        # items in {encodedvalue : originalvalue} format
        vocabKeys = {}
        # Also keep a list of integer keys, if available.
        integerKeys = {}
        for key in vocab:
            # vocabulary keys can only be strings or integers
            if isinstance(key, str):
                vocabKeys[key.decode(site_charset)] = key
            else:
                vocabKeys[key] = key
                integerKeys[key] = key

        # compile a dictonary of {encodedvalue : oldvalue} items
        # from value -- which may be a sequence, string or integer.
        pos = 0
        values = {}
        if isinstance(value, tuple) or isinstance(value, list):
            for v in value:
                new = v
                if isinstance(v, int):
                    v = str(v)
                elif isinstance(v, str):
                    new = v.decode(site_charset)
                values[(new, pos)] = v
                pos += 1
        else:
            if isinstance(value, str):
                new = value.decode(site_charset)
            elif isinstance(value, int):
                new = value
            else:
                new = str(value)
            values[(new, pos)] = value

        # now, build a list of the vocabulary keys
        # in their original charsets.
        selected = []
        for v, pos in values:
            ov = vocabKeys.get(v)
            if ov is not None:
                selected.append((pos, ov))
            elif integerKeys:
                # Submitting a string '5' where the vocabulary has
                # only integer keys works fine when the edit succeeds.
                # But when the edit form gets redisplayed (e.g. due to
                # a missing required field), the string '5' means
                # nothing is selected here, so you loose what you
                # filled in.  This gets fixed right here.
                try:
                    int_value = int(value)
                except (ValueError, TypeError):
                    continue
                ov = integerKeys.get(int_value)
                if ov is not None:
                    selected.append((pos, ov))
        selected.sort()
        return [v for pos, v in selected]


class TextareaWidget(BrowserView):
    """View used in Archetypes textarea widget."""

    def getSelected(self, mimetypes, contenttype):
        # An object can have only one contenttype at a time and mimetypes
        # are limited to ASCII-only characters. We already assumed to get all
        # values in all lowercase, so we don't do any case-juggling.

        contenttype = MIMETYPES_MAPPING.get(contenttype, contenttype)

        if contenttype in mimetypes:
            return (contenttype, )

        return ()

    def lookupMime(self, name):
        context = aq_inner(self.context)
        mimetool = context.mimetypes_registry
        mimetypes = mimetool.lookup(name)
        if len(mimetypes):
            return mimetypes[0].name()
        else:
            return name



## Script (Python) "session_restore_value"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=fieldName,default
##title=Return field value previously stored on session by session_save_form
##

return default



################################################################################
#
# Copyright (c) 2002-2005, Benjamin Saller <bcsaller@ideasuite.com>, and
#                              the respective authors. All rights reserved.
# For a list of Archetypes contributors see docs/CREDITS.txt.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name of the author nor the names of its contributors may be used
#   to endorse or promote products derived from this software without specific
#   prior written permission.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
################################################################################
"""
"""

import re
import sys
import logging

from zope.tal import ndiff
from App.Common import package_home
from ZPublisher.HTTPRequest import HTTPRequest
from ZPublisher.HTTPResponse import HTTPResponse

from Products.Archetypes.atapi import registerType
from Products.Archetypes.atapi import process_types
from Products.Archetypes.atapi import listTypes
from Products.Archetypes.atapi import BaseContent
from Products.Archetypes.config import PKG_NAME

logger = logging.getLogger('Archetypes')

PACKAGE_HOME = package_home(globals())


def gen_class(klass, schema=None):
    """generats and registers the klass
    """
    if schema is not None:
        klass.schema = schema.copy()
    registerType(klass, 'Archetypes')
    content_types, constructors, ftis = process_types(listTypes(), PKG_NAME)


def mkDummyInContext(klass, oid, context, schema=None):
    gen_class(klass, schema)
    dummy = klass(oid=oid).__of__(context)
    context._setObject(oid, dummy, suppress_events=True)
    dummy.initializeArchetype()
    return dummy


def makeContent(container, portal_type, id='document', **kw):
    container.invokeFactory(type_name=portal_type, id=id)
    return getattr(container, id)


class Dummy(BaseContent):
    def Title(self):
        return 'title'


def normalize_html(s):
    s = re.sub(r"\s+", " ", s)
    s = re.sub(r"(?s)\s+<", "<", s)
    s = re.sub(r"(?s)>\s+", ">", s)
    return s


def nicerange(lo, hi):
    if hi <= lo + 1:
        return str(lo + 1)
    else:
        return "%d,%d" % (lo + 1, hi)


def showdiff(a, b):
    cruncher = ndiff.SequenceMatcher(ndiff.IS_LINE_JUNK, a, b)
    for tag, alo, ahi, blo, bhi in cruncher.get_opcodes():
        if tag == "equal":
            continue
        print nicerange(alo, ahi) + tag[0] + nicerange(blo, bhi)
        ndiff.dump('<', a, alo, ahi)
        if a and b:
            print '---'
        ndiff.dump('>', b, blo, bhi)


def populateFolder(folder, folder_type, doc_type):
    """ Creates a structure like:

    \index_html
    \doc1
    \folder1
       \folder11
       \folder12
       \doc11
    \folder2
       \folder21
       \doc21
       \index_html
       \folder22
          \folder221
             \doc2211
             \doc2212
          \folder222
             \doc2221
             \doc2222
          \folder223
             \doc2231
             \doc2232
    """
    folder.invokeFactory(doc_type, id='index_html')
    folder.invokeFactory(doc_type, id='doc1')
    folder.invokeFactory(folder_type, id='folder1')
    folder.invokeFactory(folder_type, id='folder2')
    f1 = folder.folder1
    f2 = folder.folder2
    f1.invokeFactory(folder_type, id='folder11')
    f1.invokeFactory(folder_type, id='folder12')
    f1.invokeFactory(doc_type, id='doc11')
    f2.invokeFactory(folder_type, id='folder21')
    f2.invokeFactory(folder_type, id='folder22')
    f2.invokeFactory(doc_type, id='doc21')
    f2.invokeFactory(doc_type, id='index_html')
    f22 = f2.folder22
    f22.invokeFactory(folder_type, id='folder221')
    f221 = f22.folder221
    f221.invokeFactory(doc_type, id='doc2211')
    f221.invokeFactory(doc_type, id='doc2212')
    f22.invokeFactory(folder_type, id='folder222')
    f222 = f22.folder222
    f222.invokeFactory(doc_type, id='doc2221')
    f222.invokeFactory(doc_type, id='doc2222')
    f22.invokeFactory(folder_type, id='folder223')
    f223 = f22.folder223
    f223.invokeFactory(doc_type, id='doc2231')
    f223.invokeFactory(doc_type, id='doc2232')

WRAPPER = '__at_is_wrapper_method__'
ORIG_NAME = '__at_original_method_name__'
def isWrapperMethod(meth):
    return getattr(meth, WRAPPER, False)


def wrap_method(klass, name, method, pattern='__at_wrapped_%s__'):
    old_method = getattr(klass, name)
    if isWrapperMethod(old_method):
        logger.warning('Wrapping already wrapped method at %s.%s' %
                       (klass.__name__, name))
    new_name = pattern % name
    setattr(klass, new_name, old_method)
    setattr(method, ORIG_NAME, new_name)
    setattr(method, WRAPPER, True)
    setattr(klass, name, method)


def unwrap_method(klass, name):
    old_method = getattr(klass, name)
    if not isWrapperMethod(old_method):
        raise ValueError, ('Trying to unwrap non-wrapped '
                           'method at %s.%s' % (klass.__name__, name))
    orig_name = getattr(old_method, ORIG_NAME)
    new_method = getattr(klass, orig_name)
    delattr(klass, orig_name)
    setattr(klass, name, new_method)


from zope.publisher.browser import BrowserView
class SimpleView(BrowserView):
    def __call__(self):
        return 'SimpleView simple output'


def aputrequest(file, content_type):
    resp = HTTPResponse(stdout=sys.stdout)
    environ = {}
    environ['SERVER_NAME'] = 'foo'
    environ['SERVER_PORT'] = '80'
    environ['REQUEST_METHOD'] = 'PUT'
    environ['CONTENT_TYPE'] = content_type
    req = HTTPRequest(stdin=file, environ=environ, response=resp)
    return req



from zope.interface import Interface

class IUtils(Interface):
    """Archetypes utils view
    """

    def translate(value):
        """Use zope.i18n to trnslate values
        Wrapper to be used in pythonscripts
        """



from zope.interface import Interface, Attribute

class ITemplateMixin(Interface):
    """Marker interface for TemplateMixin
    """

    default_view = Attribute('')
    suppl_views = Attribute('')

    def getLayout():
        """
        @return string for current layout
        """

    def getDefaultLayout():
        """
        @return string for default layout
        """

    def getTemplateFor(pt, default='base_view'):
        """
        """
